@@ -2,8 +2,8 @@
  * Tests for imgITools
  */
 
-const Ci = Components.interfaces;
-const Cc = Components.classes;
+var Ci = Components.interfaces;
+var Cc = Components.classes;
 
 
 /*
@@ -13,11 +13,7 @@ const Cc = Components.classes;
  * Call |dumpToFile(outData);| in a test to file to a file.
  */
 function dumpToFile(aData) {
-    const path = "/tmp";
-
-    var outputFile = Cc["@mozilla.org/file/local;1"].
-                     createInstance(Ci.nsILocalFile);
-    outputFile.initWithPath(path);
+    var outputFile = do_get_tempdir();
     outputFile.append("testdump.png");
 
     var outputStream = Cc["@mozilla.org/network/file-output-stream;1"].
@@ -134,7 +130,7 @@ if (!imgTools)
 // different set of reference images. nsIXULRuntime.OS doesn't seem to be
 // available in xpcshell, so we'll use this as a kludgy way to figure out if
 // we're running on Windows.
-var isWindows = ("@mozilla.org/windows-registry-key;1" in Cc);
+var isWindows = mozinfo.os == "win";
 
 
 /* ========== 1 ========== */
@@ -149,6 +145,8 @@ var imgFile = do_get_file(imgName);
 var istream = getFileInputStream(imgFile);
 do_check_eq(istream.available(), 8415);
 
+// Use decodeImageData for this test even though it's deprecated to ensure that
+// it correctly forwards to decodeImage and continues to work.
 var outParam = { value: null };
 imgTools.decodeImageData(istream, inMimeType, outParam);
 var container = outParam.value;
@@ -171,7 +169,7 @@ var encodedBytes = streamToArray(istream);
 var refName = "image1png16x16.jpg";
 var refFile = do_get_file(refName);
 istream = getFileInputStream(refFile);
-do_check_eq(istream.available(), 1078);
+do_check_eq(istream.available(), 1051);
 var referenceBytes = streamToArray(istream);
 
 // compare the encoder's output to the reference file.
@@ -209,9 +207,7 @@ imgFile = do_get_file(imgName);
 istream = getFileInputStream(imgFile);
 do_check_eq(istream.available(), 3494);
 
-outParam = {};
-imgTools.decodeImageData(istream, inMimeType, outParam);
-container = outParam.value;
+container = imgTools.decodeImage(istream, inMimeType);
 
 // It's not easy to look at the pixel values from JS, so just
 // check the container's size.
@@ -232,7 +228,7 @@ encodedBytes = streamToArray(istream);
 refName = isWindows ? "image2jpg16x16-win.png" : "image2jpg16x16.png";
 refFile = do_get_file(refName);
 istream = getFileInputStream(refFile);
-do_check_eq(istream.available(), 948);
+do_check_eq(istream.available(), 950);
 referenceBytes = streamToArray(istream);
 
 // compare the encoder's output to the reference file.
@@ -273,9 +269,7 @@ imgFile = do_get_file(imgName);
 istream = getFileInputStream(imgFile);
 do_check_eq(istream.available(), 1406);
 
-outParam = { value: null };
-imgTools.decodeImageData(istream, inMimeType, outParam);
-container = outParam.value;
+container = imgTools.decodeImage(istream, inMimeType);
 
 // It's not easy to look at the pixel values from JS, so just
 // check the container's size.
@@ -295,7 +289,7 @@ encodedBytes = streamToArray(istream);
 refName = "image3ico32x32.png";
 refFile = do_get_file(refName);
 istream = getFileInputStream(refFile);
-do_check_eq(istream.available(), 2281);
+do_check_eq(istream.available(), 2285);
 referenceBytes = streamToArray(istream);
 
 // compare the encoder's output to the reference file.
@@ -333,9 +327,7 @@ imgFile = do_get_file(imgName);
 istream = getFileInputStream(imgFile);
 do_check_eq(istream.available(), 1809);
 
-outParam = { value: null };
-imgTools.decodeImageData(istream, inMimeType, outParam);
-container = outParam.value;
+container = imgTools.decodeImage(istream, inMimeType);
 
 // It's not easy to look at the pixel values from JS, so just
 // check the container's size.
@@ -443,9 +435,7 @@ imgFile = do_get_file(imgName);
 istream = getFileInputStream(imgFile);
 do_check_eq(istream.available(), 3494);
 
-outParam = {};
-imgTools.decodeImageData(istream, inMimeType, outParam);
-container = outParam.value;
+container = imgTools.decodeImage(istream, inMimeType);
 
 // It's not easy to look at the pixel values from JS, so just
 // check the container's size.
@@ -663,9 +653,7 @@ for(var i=0; i<testData.length; ++i) {
     imgFile = do_get_file(dict["preImage"]);
     istream = getFileInputStream(imgFile);
 
-    var outParam = { value: null };
-    imgTools.decodeImageData(istream, dict["preImageMimeType"], outParam);
-    var container = outParam.value;
+    var container = imgTools.decodeImage(istream, dict["preImageMimeType"]);
 
     istream = imgTools.encodeImage(container, dict["refImageMimeType"]);
 
@@ -701,12 +689,12 @@ do_check_eq(istream.available(), 17759);
 var errsrc = "none";
 
 try {
-  outParam = { value: null };
-  imgTools.decodeImageData(istream, inMimeType, outParam);
-  container = outParam.value;
+  container = imgTools.decodeImage(istream, inMimeType);
 
-  // We should never hit this - decodeImageData throws an assertion because the
-  // image decoded doesn't have enough frames.
+  // We expect to hit an error during encoding because the ICO header of the
+  // image is fine, but the actual resources are corrupt. Since decodeImage()
+  // only performs a metadata decode, it doesn't decode far enough to realize
+  // this, but we'll find out when we do a full decode during encodeImage().
   try {
       istream = imgTools.encodeImage(container, "image/png");
   } catch (e) {
@@ -718,10 +706,29 @@ try {
   errsrc = "decode";
 }
 
-do_check_eq(errsrc, "decode");
+do_check_eq(errsrc, "encode");
 checkExpectedError(/NS_ERROR_FAILURE/, err);
 
 
+/* ========== bug 815359  ========== */
+testnum = 815359;
+testdesc = "test correct ico hotspots (bug 815359)";
+
+imgName = "bug815359.ico";
+inMimeType = "image/x-icon";
+imgFile = do_get_file(imgName);
+
+istream = getFileInputStream(imgFile);
+do_check_eq(istream.available(), 4286);
+
+container = imgTools.decodeImage(istream, inMimeType);
+
+var props = container.QueryInterface(Ci.nsIProperties);
+
+do_check_eq(props.get("hotspotX", Ci.nsISupportsPRUint32).data, 10);
+do_check_eq(props.get("hotspotY", Ci.nsISupportsPRUint32).data, 9);
+
+
 /* ========== end ========== */
 
 } catch (e) {
