@@ -21,14 +21,14 @@
 
 namespace base {
 
-#define DVLOG(x) LOG(ERROR)
+#define DVLOG(x) CHROMIUM_LOG(ERROR)
 #define CHECK_GT DCHECK_GT
 #define CHECK_LT DCHECK_LT
 typedef ::Lock Lock;
 typedef ::AutoLock AutoLock;
 
 // Static table of checksums for all possible 8 bit bytes.
-const uint32 Histogram::kCrcTable[256] = {0x0, 0x77073096L, 0xee0e612cL,
+const uint32_t Histogram::kCrcTable[256] = {0x0, 0x77073096L, 0xee0e612cL,
 0x990951baL, 0x76dc419L, 0x706af48fL, 0xe963a535L, 0x9e6495a3L, 0xedb8832L,
 0x79dcb8a4L, 0xe0d5e91eL, 0x97d2d988L, 0x9b64c2bL, 0x7eb17cbdL, 0xe7b82d07L,
 0x90bf1d91L, 0x1db71064L, 0x6ab020f2L, 0xf3b97148L, 0x84be41deL, 0x1adad47dL,
@@ -96,7 +96,6 @@ Histogram* Histogram::FactoryGet(const std::string& name,
     // Extra variable is not needed... but this keeps this section basically
     // identical to other derived classes in this file (and compiler will
     // optimize away the extra variable.
-    // To avoid racy destruction at shutdown, the following will be leaked.
     Histogram* tentative_histogram =
         new Histogram(name, minimum, maximum, bucket_count);
     tentative_histogram->InitializeBucketRange();
@@ -202,8 +201,8 @@ void Histogram::WriteAscii(bool graph_it, const std::string& newline,
     }
   }
 
-  int64 remaining = sample_count;
-  int64 past = 0;
+  int64_t remaining = sample_count;
+  int64_t past = 0;
   // Output the actual histogram graph.
   for (size_t i = 0; i < bucket_count(); ++i) {
     Count current = snapshot.counts(i);
@@ -261,7 +260,7 @@ bool Histogram::DeserializeHistogramInfo(const std::string& histogram_info) {
   int declared_min;
   int declared_max;
   size_t bucket_count;
-  uint32 range_checksum;
+  uint32_t range_checksum;
   int histogram_type;
   int pickle_flags;
   SampleSet sample;
@@ -275,7 +274,7 @@ bool Histogram::DeserializeHistogramInfo(const std::string& histogram_info) {
       !pickle.ReadInt(&iter, &histogram_type) ||
       !pickle.ReadInt(&iter, &pickle_flags) ||
       !sample.Histogram::SampleSet::Deserialize(&iter, pickle)) {
-    LOG(ERROR) << "Pickle error decoding Histogram: " << histogram_name;
+    CHROMIUM_LOG(ERROR) << "Pickle error decoding Histogram: " << histogram_name;
     return false;
   }
   DCHECK(pickle_flags & kIPCSerializationSourceFlag);
@@ -283,7 +282,7 @@ bool Histogram::DeserializeHistogramInfo(const std::string& histogram_info) {
   // checks above and beyond those in Histogram::Initialize()
   if (declared_max <= 0 || declared_min <= 0 || declared_max < declared_min ||
       INT_MAX / sizeof(Count) <= bucket_count || bucket_count < 2) {
-    LOG(ERROR) << "Values error decoding Histogram: " << histogram_name;
+    CHROMIUM_LOG(ERROR) << "Values error decoding Histogram: " << histogram_name;
     return false;
   }
 
@@ -302,8 +301,8 @@ bool Histogram::DeserializeHistogramInfo(const std::string& histogram_info) {
   } else if (histogram_type == BOOLEAN_HISTOGRAM) {
     render_histogram = BooleanHistogram::FactoryGet(histogram_name, flags);
   } else {
-    LOG(ERROR) << "Error Deserializing Histogram Unknown histogram_type: "
-               << histogram_type;
+    CHROMIUM_LOG(ERROR) << "Error Deserializing Histogram Unknown histogram_type: "
+                        << histogram_type;
     return false;
   }
 
@@ -332,7 +331,7 @@ Histogram::Inconsistencies Histogram::FindCorruption(
     const SampleSet& snapshot) const {
   int inconsistencies = NO_INCONSISTENCIES;
   Sample previous_range = -1;  // Bottom range is always 0.
-  int64 count = 0;
+  int64_t count = 0;
   for (size_t index = 0; index < bucket_count(); ++index) {
     count += snapshot.counts(index);
     int new_range = ranges(index);
@@ -344,7 +343,7 @@ Histogram::Inconsistencies Histogram::FindCorruption(
   if (!HasValidRangeChecksum())
     inconsistencies |= RANGE_CHECKSUM_ERROR;
 
-  int64 delta64 = snapshot.redundant_count() - count;
+  int64_t delta64 = snapshot.redundant_count() - count;
   if (delta64 != 0) {
     int delta = static_cast<int>(delta64);
     if (delta != delta64)
@@ -410,29 +409,49 @@ bool Histogram::HasValidRangeChecksum() const {
   return CalculateRangeChecksum() == range_checksum_;
 }
 
+size_t Histogram::SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf)
+{
+  size_t n = 0;
+  n += aMallocSizeOf(this);
+  // We're not allowed to do deep dives into STL data structures.  This
+  // is as close as we can get to measuring this array.
+  n += aMallocSizeOf(&ranges_[0]);
+  n += sample_.SizeOfExcludingThis(aMallocSizeOf);
+  return n;
+}
+
+size_t Histogram::SampleSet::SizeOfExcludingThis(mozilla::MallocSizeOf aMallocSizeOf)
+{
+  // We're not allowed to do deep dives into STL data structures.  This
+  // is as close as we can get to measuring this array.
+  return aMallocSizeOf(&counts_[0]);
+}
+
 Histogram::Histogram(const std::string& name, Sample minimum,
                      Sample maximum, size_t bucket_count)
-  : histogram_name_(name),
+  : sample_(),
+    histogram_name_(name),
     declared_min_(minimum),
     declared_max_(maximum),
     bucket_count_(bucket_count),
     flags_(kNoFlags),
     ranges_(bucket_count + 1, 0),
     range_checksum_(0),
-    sample_() {
+    recording_enabled_(true) {
   Initialize();
 }
 
 Histogram::Histogram(const std::string& name, TimeDelta minimum,
                      TimeDelta maximum, size_t bucket_count)
-  : histogram_name_(name),
+  : sample_(),
+    histogram_name_(name),
     declared_min_(static_cast<int> (minimum.InMilliseconds())),
     declared_max_(static_cast<int> (maximum.InMilliseconds())),
     bucket_count_(bucket_count),
     flags_(kNoFlags),
     ranges_(bucket_count + 1, 0),
     range_checksum_(0),
-    sample_() {
+    recording_enabled_(true) {
   Initialize();
 }
 
@@ -440,7 +459,7 @@ Histogram::~Histogram() {
   if (StatisticsRecorder::dump_on_exit()) {
     std::string output;
     WriteAscii(true, "\n", &output);
-    LOG(INFO) << output;
+    CHROMIUM_LOG(INFO) << output;
   }
 
   // Just to make sure most derived class did this properly...
@@ -541,7 +560,8 @@ const std::string Histogram::GetAsciiBucketRange(size_t i) const {
 // Update histogram data with new sample.
 void Histogram::Accumulate(Sample value, Count count, size_t index) {
   // Note locking not done in this version!!!
-  sample_.Accumulate(value, count, index);
+  sample_.AccumulateWithExponentialStats(value, count, index,
+					 flags_ & kExtendedStatisticsFlag);
 }
 
 void Histogram::SetBucketRange(size_t i, Sample value) {
@@ -559,9 +579,9 @@ bool Histogram::ValidateBucketRanges() const {
   return true;
 }
 
-uint32 Histogram::CalculateRangeChecksum() const {
+uint32_t Histogram::CalculateRangeChecksum() const {
   DCHECK_EQ(ranges_.size(), bucket_count() + 1);
-  uint32 checksum = static_cast<uint32>(ranges_.size());  // Seed checksum.
+  uint32_t checksum = static_cast<uint32_t>(ranges_.size());  // Seed checksum.
   for (size_t index = 0; index < bucket_count(); ++index)
     checksum = Crc32(checksum, ranges(index));
   return checksum;
@@ -584,14 +604,14 @@ void Histogram::Initialize() {
 
 // We generate the CRC-32 using the low order bits to select whether to XOR in
 // the reversed polynomial 0xedb88320L.  This is nice and simple, and allows us
-// to keep the quotient in a uint32.  Since we're not concerned about the nature
+// to keep the quotient in a uint32_t.  Since we're not concerned about the nature
 // of corruptions (i.e., we don't care about bit sequencing, since we are
 // handling memory changes, which are more grotesque) so we don't bother to
 // get the CRC correct for big-endian vs little-ending calculations.  All we
 // need is a nice hash, that tends to depend on all the bits of the sample, with
 // very little chance of changes in one place impacting changes in another
 // place.
-uint32 Histogram::Crc32(uint32 sum, Histogram::Sample range) {
+uint32_t Histogram::Crc32(uint32_t sum, Histogram::Sample range) {
   const bool kUseRealCrc = true;  // TODO(jar): Switch to false and watch stats.
   if (kUseRealCrc) {
     union {
@@ -607,12 +627,12 @@ uint32 Histogram::Crc32(uint32 sum, Histogram::Sample range) {
     // and we don't care about edge cases since we have an even number of bytes.
     union {
       Histogram::Sample range;
-      uint16 ints[sizeof(Histogram::Sample) / 2];
+      uint16_t ints[sizeof(Histogram::Sample) / 2];
     } converter;
     DCHECK_EQ(sizeof(Histogram::Sample), sizeof(converter));
     converter.range = range;
     sum += converter.ints[0];
-    sum = (sum << 16) ^ sum ^ (static_cast<uint32>(converter.ints[1]) << 11);
+    sum = (sum << 16) ^ sum ^ (static_cast<uint32_t>(converter.ints[1]) << 11);
     sum += sum >> 11;
   }
   return sum;
@@ -649,9 +669,9 @@ void Histogram::WriteAsciiHeader(const SampleSet& snapshot,
     StringAppendF(output, " (flags = 0x%x)", flags_ & ~kHexRangePrintingFlag);
 }
 
-void Histogram::WriteAsciiBucketContext(const int64 past,
+void Histogram::WriteAsciiBucketContext(const int64_t past,
                                         const Count current,
-                                        const int64 remaining,
+                                        const int64_t remaining,
                                         const size_t i,
                                         std::string* output) const {
   double scaled_sum = (past + current + remaining) / 100.0;
@@ -688,6 +708,9 @@ void Histogram::WriteAsciiBucketGraph(double current_size, double max_size,
 Histogram::SampleSet::SampleSet()
     : counts_(),
       sum_(0),
+      sum_squares_(0),
+      log_sum_(0),
+      log_sum_squares_(0),
       redundant_count_(0) {
 }
 
@@ -702,18 +725,37 @@ void Histogram::SampleSet::CheckSize(const Histogram& histogram) const {
   DCHECK_EQ(histogram.bucket_count(), counts_.size());
 }
 
-
-void Histogram::SampleSet::Accumulate(Sample value,  Count count,
-                                      size_t index) {
+void Histogram::SampleSet::Accumulate(Sample value, Count count,
+				      size_t index) {
   DCHECK(count == 1 || count == -1);
   counts_[index] += count;
-  sum_ += count * value;
   redundant_count_ += count;
+  sum_ += static_cast<int64_t>(count) * value;
   DCHECK_GE(counts_[index], 0);
   DCHECK_GE(sum_, 0);
   DCHECK_GE(redundant_count_, 0);
 }
 
+void Histogram::SampleSet::AccumulateWithLinearStats(Sample value,
+                                                     Count count,
+                                                     size_t index) {
+  Accumulate(value, count, index);
+  sum_squares_ += static_cast<int64_t>(count) * value * value;
+}
+
+void Histogram::SampleSet::AccumulateWithExponentialStats(Sample value,
+                                                          Count count,
+                                                          size_t index,
+							  bool computeExtendedStatistics) {
+  Accumulate(value, count, index);
+  if (computeExtendedStatistics) {
+    DCHECK_GE(value, 0);
+    float value_log = logf(static_cast<float>(value) + 1.0f);
+    log_sum_ += count * value_log;
+    log_sum_squares_ += count * value_log * value_log;
+  }
+}
+
 Count Histogram::SampleSet::TotalCount() const {
   Count total = 0;
   for (Counts::const_iterator it = counts_.begin();
@@ -727,6 +769,9 @@ Count Histogram::SampleSet::TotalCount() const {
 void Histogram::SampleSet::Add(const SampleSet& other) {
   DCHECK_EQ(counts_.size(), other.counts_.size());
   sum_ += other.sum_;
+  sum_squares_ += other.sum_squares_;
+  log_sum_ += other.log_sum_;
+  log_sum_squares_ += other.log_sum_squares_;
   redundant_count_ += other.redundant_count_;
   for (size_t index = 0; index < counts_.size(); ++index)
     counts_[index] += other.counts_[index];
@@ -738,6 +783,9 @@ void Histogram::SampleSet::Subtract(const SampleSet& other) {
   // negative values when snapshots are later combined (and deltas calculated).
   // As a result, we don't currently CHCEK() for positive values.
   sum_ -= other.sum_;
+  sum_squares_ -= other.sum_squares_;
+  log_sum_ -= other.log_sum_;
+  log_sum_squares_ -= other.log_sum_squares_;
   redundant_count_ -= other.redundant_count_;
   for (size_t index = 0; index < counts_.size(); ++index) {
     counts_[index] -= other.counts_[index];
@@ -806,7 +854,6 @@ Histogram* LinearHistogram::FactoryGet(const std::string& name,
     maximum = kSampleType_MAX - 1;
 
   if (!StatisticsRecorder::FindHistogram(name, &histogram)) {
-    // To avoid racy destruction at shutdown, the following will be leaked.
     LinearHistogram* tentative_histogram =
         new LinearHistogram(name, minimum, maximum, bucket_count);
     tentative_histogram->InitializeBucketRange();
@@ -833,6 +880,10 @@ Histogram::ClassType LinearHistogram::histogram_type() const {
   return LINEAR_HISTOGRAM;
 }
 
+void LinearHistogram::Accumulate(Sample value, Count count, size_t index) {
+  sample_.AccumulateWithLinearStats(value, count, index);
+}
+
 void LinearHistogram::SetRangeDescriptions(
     const DescriptionPair descriptions[]) {
   for (int i =0; descriptions[i].description; ++i) {
@@ -898,7 +949,6 @@ Histogram* BooleanHistogram::FactoryGet(const std::string& name, Flags flags) {
   Histogram* histogram(NULL);
 
   if (!StatisticsRecorder::FindHistogram(name, &histogram)) {
-    // To avoid racy destruction at shutdown, the following will be leaked.
     BooleanHistogram* tentative_histogram = new BooleanHistogram(name);
     tentative_histogram->InitializeBucketRange();
     tentative_histogram->SetFlags(flags);
@@ -922,6 +972,14 @@ BooleanHistogram::BooleanHistogram(const std::string& name)
     : LinearHistogram(name, 1, 2, 3) {
 }
 
+void
+BooleanHistogram::Accumulate(Sample value, Count count, size_t index)
+{
+  // Callers will have computed index based on the non-booleanified value.
+  // So we need to adjust the index manually.
+  LinearHistogram::Accumulate(!!value, count, value ? 1 : 0);
+}
+
 //------------------------------------------------------------------------------
 // FlagHistogram:
 //------------------------------------------------------------------------------
@@ -929,15 +987,14 @@ BooleanHistogram::BooleanHistogram(const std::string& name)
 Histogram *
 FlagHistogram::FactoryGet(const std::string &name, Flags flags)
 {
-  Histogram *h(nsnull);
+  Histogram *h(nullptr);
 
   if (!StatisticsRecorder::FindHistogram(name, &h)) {
-    // To avoid racy destruction at shutdown, the following will be leaked.
     FlagHistogram *fh = new FlagHistogram(name);
     fh->InitializeBucketRange();
     fh->SetFlags(flags);
     size_t zero_index = fh->BucketIndex(0);
-    fh->Histogram::Accumulate(1, 1, zero_index);
+    fh->LinearHistogram::Accumulate(0, 1, zero_index);
     h = StatisticsRecorder::RegisterOrDeleteDuplicate(fh);
   }
 
@@ -963,9 +1020,9 @@ FlagHistogram::Accumulate(Sample value, Count count, size_t index)
 
   mSwitched = true;
   DCHECK_EQ(value, 1);
-  Histogram::Accumulate(value, 1, index);
+  LinearHistogram::Accumulate(value, 1, index);
   size_t zero_index = BucketIndex(0);
-  Histogram::Accumulate(1, -1, zero_index);
+  LinearHistogram::Accumulate(0, -1, zero_index);
 }
 
 void
@@ -993,6 +1050,70 @@ FlagHistogram::AddSampleSet(const SampleSet& sample) {
     Accumulate(1, 1, one_index);
   }
 }
+
+void
+FlagHistogram::Clear() {
+  Histogram::Clear();
+
+  mSwitched = false;
+  size_t zero_index = BucketIndex(0);
+  LinearHistogram::Accumulate(0, 1, zero_index);
+}
+
+//------------------------------------------------------------------------------
+// CountHistogram:
+//------------------------------------------------------------------------------
+
+Histogram *
+CountHistogram::FactoryGet(const std::string &name, Flags flags)
+{
+  Histogram *h(nullptr);
+
+  if (!StatisticsRecorder::FindHistogram(name, &h)) {
+    CountHistogram *fh = new CountHistogram(name);
+    fh->InitializeBucketRange();
+    fh->SetFlags(flags);
+    h = StatisticsRecorder::RegisterOrDeleteDuplicate(fh);
+  }
+
+  return h;
+}
+
+CountHistogram::CountHistogram(const std::string &name)
+  : LinearHistogram(name, 1, 2, 3) {
+}
+
+Histogram::ClassType
+CountHistogram::histogram_type() const
+{
+  return COUNT_HISTOGRAM;
+}
+
+void
+CountHistogram::Accumulate(Sample value, Count count, size_t index)
+{
+  size_t zero_index = BucketIndex(0);
+  LinearHistogram::Accumulate(1, 1, zero_index);
+}
+
+void
+CountHistogram::AddSampleSet(const SampleSet& sample) {
+  DCHECK_EQ(bucket_count(), sample.size());
+  // We can't be sure the SampleSet provided came from another CountHistogram,
+  // so we at least check that the unused buckets are empty.
+
+  const size_t indices[] = { BucketIndex(0), BucketIndex(1), BucketIndex(2) };
+
+  if (sample.counts(indices[1]) != 0 || sample.counts(indices[2]) != 0) {
+    return;
+  }
+
+  if (sample.counts(indices[0]) != 0) {
+    Accumulate(1, sample.counts(indices[0]), indices[0]);
+  }
+}
+
+
 //------------------------------------------------------------------------------
 // CustomHistogram:
 //------------------------------------------------------------------------------
@@ -1016,7 +1137,6 @@ Histogram* CustomHistogram::FactoryGet(const std::string& name,
   DCHECK_LT(ranges.back(), kSampleType_MAX);
 
   if (!StatisticsRecorder::FindHistogram(name, &histogram)) {
-    // To avoid racy destruction at shutdown, the following will be leaked.
     CustomHistogram* tentative_histogram = new CustomHistogram(name, ranges);
     tentative_histogram->InitializedCustomBucketRange(ranges);
     tentative_histogram->SetFlags(flags);
@@ -1085,7 +1205,7 @@ StatisticsRecorder::~StatisticsRecorder() {
   if (dump_on_exit_) {
     std::string output;
     WriteGraph("", &output);
-    LOG(INFO) << output;
+    CHROMIUM_LOG(INFO) << output;
   }
   // Clean up.
   HistogramMap* histograms = NULL;
@@ -1093,6 +1213,13 @@ StatisticsRecorder::~StatisticsRecorder() {
     base::AutoLock auto_lock(*lock_);
     histograms = histograms_;
     histograms_ = NULL;
+    for (HistogramMap::iterator it = histograms->begin();
+         histograms->end() != it;
+         ++it) {
+      // No other clients permanently hold Histogram references, so we
+      // have the only one and it is safe to delete it.
+      delete it->second;
+    }
   }
   delete histograms;
   // We don't delete lock_ on purpose to avoid having to properly protect
