@@ -16,17 +16,18 @@
 #include "nsAutoPtr.h"
 #include "nsString.h"
 #include "nsThreadUtils.h"
+#include "mozilla/RefPtr.h"
+#include "mozilla/Snprintf.h"
 
-
-#define LOG(args...)  __android_log_print(ANDROID_LOG_INFO, "Gonk", args)
+#define NETD_LOG(args...)  __android_log_print(ANDROID_LOG_INFO, "Gonk", args)
 #define ICS_SYS_USB_RNDIS_MAC "/sys/class/android_usb/android0/f_rndis/ethaddr"
 #define INVALID_SOCKET -1
 #define MAX_RECONNECT_TIMES 10
 
 namespace {
 
-mozilla::RefPtr<mozilla::ipc::NetdClient> gNetdClient;
-mozilla::RefPtr<mozilla::ipc::NetdConsumer> gNetdConsumer;
+RefPtr<mozilla::ipc::NetdClient> gNetdClient;
+RefPtr<mozilla::ipc::NetdConsumer> gNetdConsumer;
 
 class StopNetdConsumer : public nsRunnable {
 public:
@@ -53,7 +54,7 @@ InitRndisAddress()
 
   fd.rwget() = open(ICS_SYS_USB_RNDIS_MAC, O_WRONLY);
   if (fd.rwget() == -1) {
-    LOG("Unable to open file %s.", ICS_SYS_USB_RNDIS_MAC);
+    NETD_LOG("Unable to open file %s.", ICS_SYS_USB_RNDIS_MAC);
     return false;
   }
 
@@ -67,28 +68,28 @@ InitRndisAddress()
     address[i % (kEthernetAddressLength - 1) + 1] ^= serialno[i];
   }
 
-  sprintf(mac, "%02x:%02x:%02x:%02x:%02x:%02x",
-          address[0], address[1], address[2],
-          address[3], address[4], address[5]);
+  snprintf_literal(mac, "%02x:%02x:%02x:%02x:%02x:%02x",
+                   address[0], address[1], address[2],
+                   address[3], address[4], address[5]);
   length = strlen(mac);
   ret = write(fd.get(), mac, length);
   if (ret != length) {
-    LOG("Fail to write file %s.", ICS_SYS_USB_RNDIS_MAC);
+    NETD_LOG("Fail to write file %s.", ICS_SYS_USB_RNDIS_MAC);
     return false;
   }
   return true;
 }
 
-} // anonymous namespace
+} // namespace
 
 namespace mozilla {
 namespace ipc {
 
 NetdClient::NetdClient()
-  : mSocket(INVALID_SOCKET)
+  : LineWatcher('\0', MAX_COMMAND_SIZE)
   , mIOLoop(MessageLoopForIO::current())
+  , mSocket(INVALID_SOCKET)
   , mCurrentWriteOffset(0)
-  , mReceivedIndex(0)
   , mReConnectTimes(0)
 {
   MOZ_COUNT_CTOR(NetdClient);
@@ -106,23 +107,23 @@ NetdClient::OpenSocket()
                                         ANDROID_SOCKET_NAMESPACE_RESERVED,
                                         SOCK_STREAM);
   if (mSocket.rwget() < 0) {
-    LOG("Error connecting to : netd (%s) - will retry", strerror(errno));
+    NETD_LOG("Error connecting to : netd (%s) - will retry", strerror(errno));
     return false;
   }
   // Add FD_CLOEXEC flag.
   int flags = fcntl(mSocket.get(), F_GETFD);
   if (flags == -1) {
-    LOG("Error doing fcntl with F_GETFD command(%s)", strerror(errno));
+    NETD_LOG("Error doing fcntl with F_GETFD command(%s)", strerror(errno));
     return false;
   }
   flags |= FD_CLOEXEC;
   if (fcntl(mSocket.get(), F_SETFD, flags) == -1) {
-    LOG("Error doing fcntl with F_SETFD command(%s)", strerror(errno));
+    NETD_LOG("Error doing fcntl with F_SETFD command(%s)", strerror(errno));
     return false;
   }
   // Set non-blocking.
   if (fcntl(mSocket.get(), F_SETFL, O_NONBLOCK) == -1) {
-    LOG("Error set non-blocking socket(%s)", strerror(errno));
+    NETD_LOG("Error set non-blocking socket(%s)", strerror(errno));
     return false;
   }
   if (!MessageLoopForIO::current()->
@@ -131,7 +132,7 @@ NetdClient::OpenSocket()
                           MessageLoopForIO::WATCH_READ,
                           &mReadWatcher,
                           this)) {
-    LOG("Error set socket read watcher(%s)", strerror(errno));
+    NETD_LOG("Error set socket read watcher(%s)", strerror(errno));
     return false;
   }
 
@@ -144,67 +145,28 @@ NetdClient::OpenSocket()
                           this);
   }
 
-  LOG("Connected to netd");
+  NETD_LOG("Connected to netd");
   return true;
 }
 
-void
-NetdClient::OnFileCanReadWithoutBlocking(int aFd)
+void NetdClient::OnLineRead(int aFd, nsDependentCSubstring& aMessage)
 {
-  ssize_t length = 0;
+  // Set errno to 0 first. For preventing to use the stale version of errno.
+  errno = 0;
+  // We found a line terminator. Each line is formatted as an
+  // integer response code followed by the rest of the line.
+  // Fish out the response code.
+  int responseCode = strtol(aMessage.Data(), nullptr, 10);
+  if (!errno) {
+    NetdCommand* response = new NetdCommand();
+    // Passing all the response message, including the line terminator.
+    response->mSize = aMessage.Length();
+    memcpy(response->mData, aMessage.Data(), aMessage.Length());
+    gNetdConsumer->MessageReceived(response);
+  }
 
-  MOZ_ASSERT(aFd == mSocket.get());
-  while (true) {
-    errno = 0;
-    MOZ_ASSERT(mReceivedIndex < MAX_COMMAND_SIZE);
-    length = read(aFd, &mReceiveBuffer[mReceivedIndex], MAX_COMMAND_SIZE - mReceivedIndex);
-    MOZ_ASSERT(length <= ssize_t(MAX_COMMAND_SIZE - mReceivedIndex));
-    if (length <= 0) {
-      if (length == -1) {
-        if (errno == EINTR) {
-          continue; // retry system call when interrupted
-        }
-        if (errno == EAGAIN || errno == EWOULDBLOCK) {
-          return; // no data available: return and re-poll
-        }
-      }
-      LOG("Can't read from netd error: %d (%s) length: %d", errno, strerror(errno), length);
-      // At this point, assume that we can't actually access
-      // the socket anymore, and start a reconnect loop.
-      Restart();
-      return;
-    }
-
-    while (length-- > 0) {
-      MOZ_ASSERT(mReceivedIndex < MAX_COMMAND_SIZE);
-      if (mReceiveBuffer[mReceivedIndex] == '\0') {
-        // We found a line terminator. Each line is formatted as an
-        // integer response code followed by the rest of the line.
-        // Fish out the response code.
-        errno = 0;
-        int responseCode = strtol(mReceiveBuffer, nullptr, 10);
-        // TODO, Bug 783966, handle InterfaceChange(600) and BandwidthControl(601).
-        if (!errno && responseCode < 600) {
-          NetdCommand* response = new NetdCommand();
-          // Passing all the response message, including the line terminator.
-          response->mSize = mReceivedIndex + 1;
-          memcpy(response->mData, mReceiveBuffer, mReceivedIndex + 1);
-          gNetdConsumer->MessageReceived(response);
-        }
-        if (!responseCode || errno) {
-          LOG("Can't parse netd's response: %d (%s)", errno, strerror(errno));
-        }
-        // There is data in the receive buffer beyond the current line.
-        // Shift it down to the beginning.
-        if (length > 0) {
-          MOZ_ASSERT(mReceivedIndex < (MAX_COMMAND_SIZE - 1));
-          memmove(&mReceiveBuffer[0], &mReceiveBuffer[mReceivedIndex + 1], length);
-        }
-        mReceivedIndex = 0;
-      } else {
-        mReceivedIndex++;
-      }
-    }
+  if (!responseCode) {
+    NETD_LOG("Can't parse netd's response");
   }
 }
 
@@ -216,7 +178,7 @@ NetdClient::OnFileCanWriteWithoutBlocking(int aFd)
 }
 
 void
-NetdClient::Restart()
+NetdClient::OnError()
 {
   MOZ_ASSERT(MessageLoop::current() == XRE_GetIOMessageLoop());
 
@@ -224,7 +186,6 @@ NetdClient::Restart()
   mWriteWatcher.StopWatchingFileDescriptor();
 
   mSocket.dispose();
-  mReceivedIndex = 0;
   mCurrentWriteOffset = 0;
   mCurrentNetdCommand = nullptr;
   while (!mOutgoingQ.empty()) {
@@ -241,15 +202,15 @@ NetdClient::Start()
   MOZ_ASSERT(MessageLoop::current() == XRE_GetIOMessageLoop());
 
   if (!gNetdClient) {
-    LOG("Netd Client is not initialized");
+    NETD_LOG("Netd Client is not initialized");
     return;
   }
 
   if (!gNetdClient->OpenSocket()) {
     // Socket open failed, try again in a second.
-    LOG("Fail to connect to Netd");
+    NETD_LOG("Fail to connect to Netd");
     if (++gNetdClient->mReConnectTimes > MAX_RECONNECT_TIMES) {
-      LOG("Fail to connect to Netd after retry %d times", MAX_RECONNECT_TIMES);
+      NETD_LOG("Fail to connect to Netd after retry %d times", MAX_RECONNECT_TIMES);
       return;
     }
 
@@ -270,14 +231,14 @@ NetdClient::SendNetdCommandIOThread(NetdCommand* aMessage)
   MOZ_ASSERT(aMessage);
 
   if (!gNetdClient) {
-    LOG("Netd Client is not initialized");
+    NETD_LOG("Netd Client is not initialized");
     return;
   }
 
   gNetdClient->mOutgoingQ.push(aMessage);
 
   if (gNetdClient->mSocket.get() == INVALID_SOCKET) {
-    LOG("Netd connection is not established, push the message to queue");
+    NETD_LOG("Netd connection is not established, push the message to queue");
     return;
   }
 
@@ -299,8 +260,8 @@ NetdClient::WriteNetdCommand()
                             mCurrentNetdCommand->mData + mCurrentWriteOffset,
                             write_amount);
     if (written < 0) {
-      LOG("Cannot write to network, error %d\n", (int) written);
-      Restart();
+      NETD_LOG("Cannot write to network, error %d\n", (int) written);
+      OnError();
       return;
     }
 
@@ -309,7 +270,7 @@ NetdClient::WriteNetdCommand()
     }
 
     if (written != write_amount) {
-      LOG("WriteNetdCommand fail !!! Write is not completed");
+      NETD_LOG("WriteNetdCommand fail !!! Write is not completed");
       break;
     }
   }
@@ -344,7 +305,7 @@ InitNetdIOThread()
     // usb tethering only. Others service such as wifi tethering still need
     // to use ipc to communicate with netd.
     if (!result) {
-      LOG("fail to give rndis interface an address");
+      NETD_LOG("fail to give rndis interface an address");
     }
   }
   gNetdClient = new NetdClient();
