@@ -4,9 +4,9 @@
 
 this.EXPORTED_SYMBOLS = ["WeaveCrypto"];
 
-const Cc = Components.classes;
-const Ci = Components.interfaces;
-const Cr = Components.results;
+var Cc = Components.classes;
+var Ci = Components.interfaces;
+var Cr = Components.results;
 
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 Components.utils.import("resource://gre/modules/Services.jsm");
@@ -57,7 +57,11 @@ WeaveCrypto.prototype = {
             this.prefBranch = Services.prefs.getBranch("services.sync.log.");
             this.prefBranch.addObserver("cryptoDebug", this.observer, false);
             this.observer._self = this;
-            this.debug = this.prefBranch.getBoolPref("cryptoDebug");
+            try {
+              this.debug = this.prefBranch.getBoolPref("cryptoDebug");
+            } catch (x) {
+              this.debug = false;
+            }
 
             this.initNSS();
             this.initAlgorithmSettings();   // Depends on NSS.
@@ -128,17 +132,16 @@ WeaveCrypto.prototype = {
 
         // XXX really want to be able to pass specific dlopen flags here.
         var nsslib;
-        try {
-            this.log("Trying NSS library without path");
-            nsslib = ctypes.open(path);
-        } catch(e) {
-            // In case opening the library without a full path fails,
-            // try again with a full path.
-            let file = Services.dirsvc.get("GreD", Ci.nsILocalFile);
-            file.append(path);
-            this.log("Trying again with path " + file.path);
-            nsslib = ctypes.open(file.path);
-        }
+#ifdef MOZ_NATIVE_NSS
+        // Search platform-dependent library paths for system NSS.
+        this.log("Trying NSS library without path");
+        nsslib = ctypes.open(path);
+#else
+        let file = Services.dirsvc.get("GreBinD", Ci.nsILocalFile);
+        file.append(path);
+        this.log("Trying NSS library with path " + file.path);
+        nsslib = ctypes.open(file.path);
+#endif
 
         this.log("Initializing NSS types and function declarations...");
 
@@ -535,7 +538,9 @@ WeaveCrypto.prototype = {
         }
     },
 
-    generateRandomIV : function() this.generateRandomBytes(this.ivLength),
+    generateRandomIV : function() {
+      return this.generateRandomBytes(this.ivLength);
+    },
 
     generateRandomBytes : function(byteCount) {
         this.log("generateRandomBytes() called");
@@ -705,7 +710,7 @@ WeaveCrypto.prototype = {
         // Callee picks if SEC_OID_UNKNOWN, but only SHA1 is supported.
         let prfAlg    = this.nss.SEC_OID_HMAC_SHA1;
 
-        let keyLength  = keyLength || 0;    // 0 = Callee will pick.
+        keyLength  = keyLength || 0;    // 0 = Callee will pick.
         let iterations = KEY_DERIVATION_ITERATIONS;
 
         let algid, slot, symKey, keyData;
