@@ -17,10 +17,9 @@
 
 "use strict";
 
-const Cu = Components.utils;
+var Cu = Components.utils;
 
-Cu.import("resource://services-common/log4moz.js");
-Cu.import("resource://services-sync/engines.js");
+Cu.import("resource://gre/modules/Log.jsm");
 Cu.import("resource://services-sync/util.js");
 Cu.import("resource://gre/modules/AddonManager.jsm");
 
@@ -65,8 +64,7 @@ this.EXPORTED_SYMBOLS = ["AddonsReconciler", "CHANGE_INSTALLED",
  * When you are finished with the instance, please call:
  *
  *   reconciler.stopListening();
- *   reconciler.saveStateFile(...);
- *
+ *   reconciler.saveState(...);
  *
  * There are 2 classes of listeners in the AddonManager: AddonListener and
  * InstallListener. This class is a listener for both (member functions just
@@ -115,25 +113,34 @@ this.EXPORTED_SYMBOLS = ["AddonsReconciler", "CHANGE_INSTALLED",
  * heed them like they were normal. In the end, the state is proper.
  */
 this.AddonsReconciler = function AddonsReconciler() {
-  this._log = Log4Moz.repository.getLogger("Sync.AddonsReconciler");
+  this._log = Log.repository.getLogger("Sync.AddonsReconciler");
   let level = Svc.Prefs.get("log.logger.addonsreconciler", "Debug");
-  this._log.level = Log4Moz.Level[level];
+  this._log.level = Log.Level[level];
 
-  Svc.Obs.add("weave:engine:start-tracking", this.startListening, this);
-  Svc.Obs.add("weave:engine:stop-tracking", this.stopListening, this);
   Svc.Obs.add("xpcom-shutdown", this.stopListening, this);
 };
 AddonsReconciler.prototype = {
   /** Flag indicating whether we are listening to AddonManager events. */
   _listening: false,
 
-  /** Whether state has been loaded from a file.
+  /**
+   * Whether state has been loaded from a file.
    *
    * State is loaded on demand if an operation requires it.
    */
   _stateLoaded: false,
 
-  /** log4moz logger instance */
+  /**
+   * Define this as false if the reconciler should not persist state
+   * to disk when handling events.
+   *
+   * This allows test code to avoid spinning to write during observer
+   * notifications and xpcom shutdown, which appears to cause hangs on WinXP
+   * (Bug 873861).
+   */
+  _shouldPersist: true,
+
+  /** Log logger instance */
   _log: null,
 
   /**
@@ -211,11 +218,12 @@ AddonsReconciler.prototype = {
       }
 
       this._addons = json.addons;
-      for each (let record in this._addons) {
+      for (let id in this._addons) {
+        let record = this._addons[id];
         record.modified = new Date(record.modified);
       }
 
-      for each (let [time, change, id] in json.changes) {
+      for (let [time, change, id] of json.changes) {
         this._changes.push([new Date(time), change, id]);
       }
 
@@ -251,7 +259,7 @@ AddonsReconciler.prototype = {
       }
     }
 
-    for each (let [time, change, id] in this._changes) {
+    for (let [time, change, id] of this._changes) {
       state.changes.push([time.getTime(), change, id]);
     }
 
@@ -301,8 +309,7 @@ AddonsReconciler.prototype = {
    * This is typically called automatically when Sync is loaded.
    */
   startListening: function startListening() {
-    let engine = Engines.get("addons");
-    if (!engine || !engine.enabled || this._listening) {
+    if (this._listening) {
       return;
     }
 
@@ -344,7 +351,7 @@ AddonsReconciler.prototype = {
     AddonManager.getAllAddons(function (addons) {
       let ids = {};
 
-      for each (let addon in addons) {
+      for (let addon of addons) {
         ids[addon.id] = true;
         this.rectifyStateFromAddon(addon);
       }
@@ -367,7 +374,7 @@ AddonsReconciler.prototype = {
         }
 
         let installFound = false;
-        for each (let install in installs) {
+        for (let install of installs) {
           if (install.addon && install.addon.id == id &&
               install.state == AddonManager.STATE_INSTALLED) {
 
@@ -388,7 +395,12 @@ AddonsReconciler.prototype = {
         }
       }
 
-      this.saveState(null, callback);
+      // See note for _shouldPersist.
+      if (this._shouldPersist) {
+        this.saveState(null, callback);
+      } else {
+        callback();
+      }
     }.bind(this));
   },
 
@@ -472,7 +484,7 @@ AddonsReconciler.prototype = {
     this._log.info("Change recorded for " + state.id);
     this._changes.push([date, change, state.id]);
 
-    for each (let listener in this._listeners) {
+    for (let listener of this._listeners) {
       try {
         listener.changeListener.call(listener, date, change, state);
       } catch (ex) {
@@ -543,7 +555,8 @@ AddonsReconciler.prototype = {
    * @return Object on success on null on failure.
    */
   getAddonStateFromSyncGUID: function getAddonStateFromSyncGUID(guid) {
-    for each (let addon in this.addons) {
+    for (let id in this.addons) {
+      let addon = this.addons[id];
       if (addon.guid == guid) {
         return addon;
       }
@@ -614,9 +627,12 @@ AddonsReconciler.prototype = {
           }
       }
 
-      let cb = Async.makeSpinningCallback();
-      this.saveState(null, cb);
-      cb.wait();
+      // See note for _shouldPersist.
+      if (this._shouldPersist) {
+        let cb = Async.makeSpinningCallback();
+        this.saveState(null, cb);
+        cb.wait();
+      }
     }
     catch (ex) {
       this._log.warn("Exception: " + Utils.exceptionStr(ex));
