@@ -6,7 +6,6 @@
 #import <Cocoa/Cocoa.h>
 
 #include "nsFilePicker.h"
-#include "nsObjCExceptions.h"
 #include "nsCOMPtr.h"
 #include "nsReadableUtils.h"
 #include "nsNetUtil.h"
@@ -20,6 +19,9 @@
 #include "nsCocoaUtils.h"
 #include "mozilla/Preferences.h"
 
+// This must be included last:
+#include "nsObjCExceptions.h"
+
 using namespace mozilla;
 
 const float kAccessoryViewPadding = 5;
@@ -43,7 +45,7 @@ const char kShowHiddenFilesPref[] = "filepicker.showHiddenFiles";
 - (void) menuChangedItem:(NSNotification*)aSender;
 @end
 
-NS_IMPL_ISUPPORTS1(nsFilePicker, nsIFilePicker)
+NS_IMPL_ISUPPORTS(nsFilePicker, nsIFilePicker)
 
 // We never want to call the secret show hidden files API unless the pref
 // has been set. Once the pref has been set we always need to call it even
@@ -90,8 +92,7 @@ static void SetShowHiddenFileState(NSSavePanel* panel)
 }
 
 nsFilePicker::nsFilePicker()
-: mMode(0)
-, mSelectedTypeIndex(0)
+: mSelectedTypeIndex(0)
 {
 }
 
@@ -100,11 +101,9 @@ nsFilePicker::~nsFilePicker()
 }
 
 void
-nsFilePicker::InitNative(nsIWidget *aParent, const nsAString& aTitle,
-                         PRInt16 aMode)
+nsFilePicker::InitNative(nsIWidget *aParent, const nsAString& aTitle)
 {
   mTitle = aTitle;
-  mMode = aMode;
 }
 
 NSView* nsFilePicker::GetAccessoryView()
@@ -122,10 +121,11 @@ NSView* nsFilePicker::GetAccessoryView()
   nsresult rv = sbs->CreateBundle("chrome://global/locale/filepicker.properties", getter_AddRefs(bundle));
   if (NS_SUCCEEDED(rv)) {
     nsXPIDLString locaLabel;
-    bundle->GetStringFromName(NS_LITERAL_STRING("formatLabel").get(),
+    bundle->GetStringFromName(MOZ_UTF16("formatLabel"),
 			      getter_Copies(locaLabel));
     if (locaLabel) {
-      label = [NSString stringWithCharacters:locaLabel.get() length:locaLabel.Length()];
+      label = [NSString stringWithCharacters:reinterpret_cast<const unichar*>(locaLabel.get())
+                                      length:locaLabel.Length()];
     }
   }
 
@@ -143,23 +143,23 @@ NSView* nsFilePicker::GetAccessoryView()
 
   // set up popup button
   NSPopUpButton* popupButton = [[[NSPopUpButton alloc] initWithFrame:NSMakeRect(0, 0, 0, 0) pullsDown:NO] autorelease];
-  PRUint32 numMenuItems = mTitles.Length();
-  for (PRUint32 i = 0; i < numMenuItems; i++) {
+  uint32_t numMenuItems = mTitles.Length();
+  for (uint32_t i = 0; i < numMenuItems; i++) {
     const nsString& currentTitle = mTitles[i];
     NSString *titleString;
     if (currentTitle.IsEmpty()) {
       const nsString& currentFilter = mFilters[i];
-      titleString = [[NSString alloc] initWithCharacters:currentFilter.get()
+      titleString = [[NSString alloc] initWithCharacters:reinterpret_cast<const unichar*>(currentFilter.get())
                                                   length:currentFilter.Length()];
     }
     else {
-      titleString = [[NSString alloc] initWithCharacters:currentTitle.get()
+      titleString = [[NSString alloc] initWithCharacters:reinterpret_cast<const unichar*>(currentTitle.get())
                                                   length:currentTitle.Length()];
     }
     [popupButton addItemWithTitle:titleString];
     [titleString release];
   }
-  if (mSelectedTypeIndex >= 0 && (PRUint32)mSelectedTypeIndex < numMenuItems)
+  if (mSelectedTypeIndex >= 0 && (uint32_t)mSelectedTypeIndex < numMenuItems)
     [popupButton selectItemAtIndex:mSelectedTypeIndex];
   [popupButton setTag:kSaveTypeControlTag];
   [popupButton sizeToFit]; // we have to do sizeToFit to get the height calculated for us
@@ -191,13 +191,13 @@ NSView* nsFilePicker::GetAccessoryView()
 }
 
 // Display the file dialog
-NS_IMETHODIMP nsFilePicker::Show(PRInt16 *retval)
+NS_IMETHODIMP nsFilePicker::Show(int16_t *retval)
 {
   NS_ENSURE_ARG_POINTER(retval);
 
   *retval = returnCancel;
 
-  PRInt16 userClicksOK = returnCancel;
+  int16_t userClicksOK = returnCancel;
 
 // Random questions from DHH:
 //
@@ -270,7 +270,7 @@ void UpdatePanelFileTypes(NSOpenPanel* aPanel, NSArray* aFilters)
 - (void) menuChangedItem:(NSNotification *)aSender
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_RETURN;
-  PRInt32 selectedItem = [mPopUpButton indexOfSelectedItem];
+  int32_t selectedItem = [mPopUpButton indexOfSelectedItem];
   if (selectedItem < 0) {
     return;
   }
@@ -283,12 +283,12 @@ void UpdatePanelFileTypes(NSOpenPanel* aPanel, NSArray* aFilters)
 @end
 
 // Use OpenPanel to do a GetFile. Returns |returnOK| if the user presses OK in the dialog. 
-PRInt16
+int16_t
 nsFilePicker::GetLocalFiles(const nsString& inTitle, bool inAllowMultiple, nsCOMArray<nsIFile>& outFiles)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_RETURN;
 
-  PRInt16 retVal = (PRInt16)returnCancel;
+  int16_t retVal = (int16_t)returnCancel;
   NSOpenPanel *thePanel = [NSOpenPanel openPanel];
 
   SetShowHiddenFileState(thePanel);
@@ -299,7 +299,7 @@ nsFilePicker::GetLocalFiles(const nsString& inTitle, bool inAllowMultiple, nsCOM
   [thePanel setCanSelectHiddenExtension:YES];
   [thePanel setCanChooseDirectories:NO];
   [thePanel setCanChooseFiles:YES];
-  [thePanel setResolvesAliases:YES];        //this is default - probably doesn't need to be set
+  [thePanel setResolvesAliases:YES]; //this is default - probably doesn't need to be set
   
   // Get filters
   // filters may be null, if we should allow all file types.
@@ -310,20 +310,22 @@ nsFilePicker::GetLocalFiles(const nsString& inTitle, bool inAllowMultiple, nsCOM
   
   // if this is the "Choose application..." dialog, and no other start
   // dir has been set, then use the Applications folder.
-  if (!theDir && filters && [filters count] == 1 && 
-      [(NSString *)[filters objectAtIndex:0] isEqualToString:@"app"]) {
-    theDir = @"/Applications/";
+  if (!theDir) {
+    if (filters && [filters count] == 1 &&
+        [(NSString *)[filters objectAtIndex:0] isEqualToString:@"app"])
+      theDir = @"/Applications/";
+    else
+      theDir = @"";
+  }
+
+  if (theDir) {
+    [thePanel setDirectoryURL:[NSURL fileURLWithPath:theDir isDirectory:YES]];
   }
 
-  // On 10.6+, we let users change the filters. Unfortunately, some methods
-  // are not available on 10.5 and without using them it happens to be buggy.
   int result;
   nsCocoaUtils::PrepareForNativeAppModalDialog();
-  if (mFilters.Length() > 1 && nsCocoaFeatures::OnSnowLeopardOrLater()) {
+  if (mFilters.Length() > 1) {
     // [NSURL initWithString:] (below) throws an exception if URLString is nil.
-    if (!theDir) {
-      theDir = @"";
-    }
 
     NSPopUpButtonObserver* observer = [[NSPopUpButtonObserver alloc] init];
 
@@ -339,7 +341,6 @@ nsFilePicker::GetLocalFiles(const nsString& inTitle, bool inAllowMultiple, nsCOM
       selector:@selector(menuChangedItem:)
       name:NSMenuWillSendActionNotification object:nil];
 
-    [thePanel setDirectoryURL:[[NSURL alloc] initWithString:theDir]];
     UpdatePanelFileTypes(thePanel, filters);
     result = [thePanel runModal];
 
@@ -350,7 +351,8 @@ nsFilePicker::GetLocalFiles(const nsString& inTitle, bool inAllowMultiple, nsCOM
     if (!filters) {
       [thePanel setTreatsFilePackagesAsDirectories:YES];
     }
-    result = [thePanel runModalForDirectory:theDir file:nil types:filters];
+    [thePanel setAllowedFileTypes:filters];
+    result = [thePanel runModal];
   }
   nsCocoaUtils::CleanUpAfterNativeAppModalDialog();
   
@@ -384,13 +386,13 @@ nsFilePicker::GetLocalFiles(const nsString& inTitle, bool inAllowMultiple, nsCOM
 }
 
 // Use OpenPanel to do a GetFolder. Returns |returnOK| if the user presses OK in the dialog.
-PRInt16
+int16_t
 nsFilePicker::GetLocalFolder(const nsString& inTitle, nsIFile** outFile)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_RETURN;
   NS_ASSERTION(outFile, "this protected member function expects a null initialized out pointer");
   
-  PRInt16 retVal = (PRInt16)returnCancel;
+  int16_t retVal = (int16_t)returnCancel;
   NSOpenPanel *thePanel = [NSOpenPanel openPanel];
 
   SetShowHiddenFileState(thePanel);
@@ -409,8 +411,11 @@ nsFilePicker::GetLocalFolder(const nsString& inTitle, nsIFile** outFile)
 
   // set up default directory
   NSString *theDir = PanelDefaultDirectory();
+  if (theDir) {
+    [thePanel setDirectoryURL:[NSURL fileURLWithPath:theDir isDirectory:YES]];
+  }
   nsCocoaUtils::PrepareForNativeAppModalDialog();
-  int result = [thePanel runModalForDirectory:theDir file:nil types:nil];  
+  int result = [thePanel runModal];
   nsCocoaUtils::CleanUpAfterNativeAppModalDialog();
 
   if (result == NSFileHandlingPanelCancelButton)
@@ -435,13 +440,13 @@ nsFilePicker::GetLocalFolder(const nsString& inTitle, nsIFile** outFile)
 }
 
 // Returns |returnOK| if the user presses OK in the dialog.
-PRInt16
+int16_t
 nsFilePicker::PutLocalFile(const nsString& inTitle, const nsString& inDefaultName, nsIFile** outFile)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_RETURN;
   NS_ASSERTION(outFile, "this protected member function expects a null initialized out pointer");
 
-  PRInt16 retVal = returnCancel;
+  int16_t retVal = returnCancel;
   NSSavePanel *thePanel = [NSSavePanel savePanel];
 
   SetShowHiddenFileState(thePanel);
@@ -457,10 +462,14 @@ nsFilePicker::PutLocalFile(const nsString& inTitle, const nsString& inDefaultNam
 
   // set up default directory
   NSString *theDir = PanelDefaultDirectory();
+  if (theDir) {
+    [thePanel setDirectoryURL:[NSURL fileURLWithPath:theDir isDirectory:YES]];
+  }
 
   // load the panel
   nsCocoaUtils::PrepareForNativeAppModalDialog();
-  int result = [thePanel runModalForDirectory:theDir file:defaultFilename];
+  [thePanel setNameFieldStringValue:defaultFilename];
+  int result = [thePanel runModal];
   nsCocoaUtils::CleanUpAfterNativeAppModalDialog();
   if (result == NSFileHandlingPanelCancelButton)
     return retVal;
@@ -502,7 +511,7 @@ nsFilePicker::GetFilterList()
     return nil;
   }
 
-  if (mFilters.Length() <= (PRUint32)mSelectedTypeIndex) {
+  if (mFilters.Length() <= (uint32_t)mSelectedTypeIndex) {
     NS_WARNING("An out of range index has been selected. Using the first index instead.");
     mSelectedTypeIndex = 0;
   }
@@ -519,10 +528,9 @@ nsFilePicker::GetFilterList()
   // The extensions in filterWide are in the format "*.ext" but are expected
   // in the format "ext" by NSOpenPanel. So we need to filter some characters.
   NSMutableString* filterString = [[[NSMutableString alloc] initWithString:
-                                    [NSString stringWithCharacters:filterWide.get()
-				              length:filterWide.Length()]] autorelease];
-  NSCharacterSet *set = [NSCharacterSet characterSetWithCharactersInString:
-                          [NSString stringWithString:@". *"]];
+                                    [NSString stringWithCharacters:reinterpret_cast<const unichar*>(filterWide.get())
+                                                            length:filterWide.Length()]] autorelease];
+  NSCharacterSet *set = [NSCharacterSet characterSetWithCharactersInString:@". *"];
   NSRange range = [filterString rangeOfCharacterFromSet:set];
   while (range.length) {
     [filterString replaceCharactersInRange:range withString:@""];
@@ -558,7 +566,8 @@ nsFilePicker::PanelDefaultDirectory()
   if (mDisplayDirectory) {
     nsAutoString pathStr;
     mDisplayDirectory->GetPath(pathStr);
-    directory = [[[NSString alloc] initWithCharacters:pathStr.get() length:pathStr.Length()] autorelease];
+    directory = [[[NSString alloc] initWithCharacters:reinterpret_cast<const unichar*>(pathStr.get())
+                                               length:pathStr.Length()] autorelease];
   }
   return directory;
 
@@ -568,7 +577,7 @@ nsFilePicker::PanelDefaultDirectory()
 NS_IMETHODIMP nsFilePicker::GetFile(nsIFile **aFile)
 {
   NS_ENSURE_ARG_POINTER(aFile);
-  *aFile = nsnull;
+  *aFile = nullptr;
   
   // just return the first file
   if (mFiles.Count() > 0) {
@@ -582,7 +591,7 @@ NS_IMETHODIMP nsFilePicker::GetFile(nsIFile **aFile)
 NS_IMETHODIMP nsFilePicker::GetFileURL(nsIURI **aFileURL)
 {
   NS_ENSURE_ARG_POINTER(aFileURL);
-  *aFileURL = nsnull;
+  *aFileURL = nullptr;
 
   if (mFiles.Count() == 0)
     return NS_OK;
@@ -634,14 +643,14 @@ nsFilePicker::AppendFilter(const nsAString& aTitle, const nsAString& aFilter)
 }
 
 // Get the filter index - do we still need this?
-NS_IMETHODIMP nsFilePicker::GetFilterIndex(PRInt32 *aFilterIndex)
+NS_IMETHODIMP nsFilePicker::GetFilterIndex(int32_t *aFilterIndex)
 {
   *aFilterIndex = mSelectedTypeIndex;
   return NS_OK;
 }
 
 // Set the filter index - do we still need this?
-NS_IMETHODIMP nsFilePicker::SetFilterIndex(PRInt32 aFilterIndex)
+NS_IMETHODIMP nsFilePicker::SetFilterIndex(int32_t aFilterIndex)
 {
   mSelectedTypeIndex = aFilterIndex;
   return NS_OK;
