@@ -9,6 +9,7 @@
 #include "mozilla/FileUtils.h"
 #include "mozilla/Monitor.h"
 #include "mozilla/Preferences.h"
+#include "mozilla/ProcessPriorityManager.h"
 #include "mozilla/Services.h"
 #include "nsIObserver.h"
 #include "nsIObserverService.h"
@@ -59,7 +60,7 @@ namespace {
  * between read()s, we sleep by Wait()'ing on a monitor, which we notify on
  * shutdown.
  */
-class MemoryPressureWatcher
+class MemoryPressureWatcher final
   : public nsIRunnable
   , public nsIObserver
 {
@@ -120,8 +121,6 @@ public:
 
 #ifdef MOZ_NUWA_PROCESS
     if (IsNuwaProcess()) {
-      NS_ASSERTION(NuwaMarkCurrentThread != nullptr,
-                   "NuwaMarkCurrentThread is undefined!");
       NuwaMarkCurrentThread(nullptr, nullptr);
     }
 #endif
@@ -169,7 +168,7 @@ public:
 
       // We use low-memory-no-forward because each process has its own watcher
       // and thus there is no need for the main process to forward this event.
-      rv = NS_DispatchMemoryPressure(MemPressure_New);
+      rv = DispatchMemoryPressure(MemPressure_New);
       NS_ENSURE_SUCCESS(rv, rv);
 
       // Manually check lowMemFd until we observe that memory pressure is over.
@@ -202,7 +201,7 @@ public:
         NS_ENSURE_SUCCESS(rv, rv);
 
         if (memoryPressure) {
-          rv = NS_DispatchMemoryPressure(MemPressure_Ongoing);
+          rv = DispatchMemoryPressure(MemPressure_Ongoing);
           NS_ENSURE_SUCCESS(rv, rv);
           continue;
         }
@@ -214,6 +213,9 @@ public:
     return NS_OK;
   }
 
+protected:
+  ~MemoryPressureWatcher() {}
+
 private:
   /**
    * Read from aLowMemFd, which we assume corresponds to the
@@ -245,6 +247,21 @@ private:
     return NS_OK;
   }
 
+  /**
+   * Dispatch the specified memory pressure event unless a high-priority
+   * process is present. If a high-priority process is present then it's likely
+   * responding to an urgent event (an incoming call or message for example) so
+   * avoid wasting CPU time responding to low-memory events.
+   */
+  nsresult DispatchMemoryPressure(MemoryPressureState state)
+  {
+    if (ProcessPriorityManager::AnyProcessHasHighPriority()) {
+      return NS_OK;
+    }
+
+    return NS_DispatchMemoryPressure(state);
+  }
+
   Monitor mMonitor;
   uint32_t mPollMS;
   bool mShuttingDown;
@@ -253,9 +270,9 @@ private:
   ScopedClose mShutdownPipeWrite;
 };
 
-NS_IMPL_ISUPPORTS2(MemoryPressureWatcher, nsIRunnable, nsIObserver);
+NS_IMPL_ISUPPORTS(MemoryPressureWatcher, nsIRunnable, nsIObserver);
 
-} // anonymous namespace
+} // namespace
 
 namespace mozilla {
 
@@ -263,12 +280,13 @@ void
 InitGonkMemoryPressureMonitoring()
 {
   // memoryPressureWatcher is held alive by the observer service.
-  nsRefPtr<MemoryPressureWatcher> memoryPressureWatcher =
+  RefPtr<MemoryPressureWatcher> memoryPressureWatcher =
     new MemoryPressureWatcher();
   NS_ENSURE_SUCCESS_VOID(memoryPressureWatcher->Init());
 
   nsCOMPtr<nsIThread> thread;
-  NS_NewThread(getter_AddRefs(thread), memoryPressureWatcher);
+  NS_NewNamedThread("MemoryPressure", getter_AddRefs(thread),
+                    memoryPressureWatcher);
 }
 
 } // namespace mozilla
