@@ -5,7 +5,7 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "mozilla/Util.h"
+#include "mozilla/ArrayUtils.h"
 
 #include "nsClipboard.h"
 #include "nsSupportsPrimitives.h"
@@ -13,7 +13,6 @@
 #include "nsReadableUtils.h"
 #include "nsXPIDLString.h"
 #include "nsPrimitiveHelpers.h"
-#include "nsICharsetConverterManager.h"
 #include "nsIServiceManager.h"
 #include "nsImageToPixbuf.h"
 #include "nsStringStream.h"
@@ -34,6 +33,10 @@
 #include <errno.h>
 #include <unistd.h>
 
+#include "mozilla/dom/EncodingUtils.h"
+#include "nsIUnicodeDecoder.h"
+
+using mozilla::dom::EncodingUtils;
 using namespace mozilla;
 
 // Callback when someone asks us for the data
@@ -51,7 +54,7 @@ clipboard_clear_cb(GtkClipboard *aGtkClipboard,
 static void
 ConvertHTMLtoUCS2          (guchar             *data,
                             int32_t             dataLength,
-                            PRUnichar         **unicodeData,
+                            char16_t         **unicodeData,
                             int32_t            &outUnicodeLen);
 
 static void
@@ -69,6 +72,11 @@ wait_for_contents          (GtkClipboard *clipboard, GdkAtom target);
 static gchar *
 wait_for_text              (GtkClipboard *clipboard);
 
+static GdkFilterReturn
+selection_request_filter   (GdkXEvent *gdk_xevent,
+                            GdkEvent *event,
+                            gpointer data);
+
 nsClipboard::nsClipboard()
 {
 }
@@ -85,7 +93,7 @@ nsClipboard::~nsClipboard()
     }
 }
 
-NS_IMPL_ISUPPORTS1(nsClipboard, nsIClipboard)
+NS_IMPL_ISUPPORTS(nsClipboard, nsIClipboard)
 
 nsresult
 nsClipboard::Init(void)
@@ -96,15 +104,23 @@ nsClipboard::Init(void)
 
     os->AddObserver(this, "quit-application", false);
 
+    // A custom event filter to workaround attempting to dereference a null
+    // selection requestor in GTK3 versions before 3.11.3. See bug 1178799.
+#if (MOZ_WIDGET_GTK == 3) && defined(MOZ_X11)
+    if (gtk_check_version(3, 11, 3))
+        gdk_window_add_filter(nullptr, selection_request_filter, nullptr);
+#endif
+
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsClipboard::Observe(nsISupports *aSubject, const char *aTopic, const PRUnichar *aData)
+nsClipboard::Observe(nsISupports *aSubject, const char *aTopic, const char16_t *aData)
 {
     if (strcmp(aTopic, "quit-application") == 0) {
         // application is going to quit, save clipboard content
         Store();
+        gdk_window_remove_filter(nullptr, selection_request_filter, nullptr);
     }
     return NS_OK;
 }
@@ -134,14 +150,6 @@ nsClipboard::SetData(nsITransferable *aTransferable,
         return NS_OK;
     }
 
-    nsresult rv;
-    if (!mPrivacyHandler) {
-        rv = NS_NewClipboardPrivacyHandler(getter_AddRefs(mPrivacyHandler));
-        NS_ENSURE_SUCCESS(rv, rv);
-    }
-    rv = mPrivacyHandler->PrepareDataForClipboard(aTransferable);
-    NS_ENSURE_SUCCESS(rv, rv);
-
     // Clear out the clipboard in order to set the new data
     EmptyClipboard(aWhichClipboard);
 
@@ -151,7 +159,8 @@ nsClipboard::SetData(nsITransferable *aTransferable,
     // Get the types of supported flavors
     nsCOMPtr<nsISupportsArray> flavors;
 
-    rv = aTransferable->FlavorsTransferableCanExport(getter_AddRefs(flavors));
+    nsresult rv =
+        aTransferable->FlavorsTransferableCanExport(getter_AddRefs(flavors));
     if (!flavors || NS_FAILED(rv))
         return NS_ERROR_FAILURE;
 
@@ -324,7 +333,7 @@ nsClipboard::GetData(nsITransferable *aTransferable, int32_t aWhichClipboard)
                 length = gtk_selection_data_get_length(selectionData);
                 // Special case text/html since we can convert into UCS2
                 if (!strcmp(flavorStr, kHTMLMime)) {
-                    PRUnichar* htmlBody= nullptr;
+                    char16_t* htmlBody= nullptr;
                     int32_t htmlBodyLen = 0;
                     // Convert text/html into our unicode format
                     ConvertHTMLtoUCS2(const_cast<guchar*>(clipboardData), length,
@@ -335,7 +344,7 @@ nsClipboard::GetData(nsITransferable *aTransferable, int32_t aWhichClipboard)
                     data = (guchar *)htmlBody;
                     length = htmlBodyLen * 2;
                 } else {
-                    data = (guchar *)nsMemory::Alloc(length);
+                    data = (guchar *)moz_xmalloc(length);
                     if (!data)
                         break;
                     memcpy(data, clipboardData, length);
@@ -358,7 +367,7 @@ nsClipboard::GetData(nsITransferable *aTransferable, int32_t aWhichClipboard)
     }
 
     if (data)
-        nsMemory::Free(data);
+        free(data);
 
     return NS_OK;
 }
@@ -448,6 +457,13 @@ nsClipboard::SupportsSelectionClipboard(bool *_retval)
     return NS_OK;
 }
 
+NS_IMETHODIMP
+nsClipboard::SupportsFindClipboard(bool* _retval)
+{
+  *_retval = false;
+  return NS_OK;
+}
+
 /* static */
 GdkAtom
 nsClipboard::GetSelectionAtom(int32_t aWhichClipboard)
@@ -513,9 +529,8 @@ nsClipboard::SelectionGetEvent(GtkClipboard     *aClipboard,
     nsCOMPtr<nsISupports> item;
     uint32_t len;
 
-  
     GdkAtom selectionTarget = gtk_selection_data_get_target(aSelectionData);
-  
+
     // Check to see if the selection data includes any of the string
     // types that we support.
     if (selectionTarget == gdk_atom_intern ("STRING", FALSE) ||
@@ -544,7 +559,7 @@ nsClipboard::SelectionGetEvent(GtkClipboard     *aClipboard,
         gtk_selection_data_set_text (aSelectionData, utf8string,
                                      strlen(utf8string));
 
-        nsMemory::Free(utf8string);
+        free(utf8string);
         return;
     }
 
@@ -553,12 +568,11 @@ nsClipboard::SelectionGetEvent(GtkClipboard     *aClipboard,
         // Look through our transfer data for the image
         static const char* const imageMimeTypes[] = {
             kNativeImageMime, kPNGImageMime, kJPEGImageMime, kJPGImageMime, kGIFImageMime };
-        nsCOMPtr<nsISupports> item;
-        uint32_t len;
+        nsCOMPtr<nsISupports> imageItem;
         nsCOMPtr<nsISupportsInterfacePointer> ptrPrimitive;
         for (uint32_t i = 0; !ptrPrimitive && i < ArrayLength(imageMimeTypes); i++) {
-            rv = trans->GetTransferData(imageMimeTypes[i], getter_AddRefs(item), &len);
-            ptrPrimitive = do_QueryInterface(item);
+            rv = trans->GetTransferData(imageMimeTypes[i], getter_AddRefs(imageItem), &len);
+            ptrPrimitive = do_QueryInterface(imageItem);
         }
         if (!ptrPrimitive)
             return;
@@ -606,13 +620,13 @@ nsClipboard::SelectionGetEvent(GtkClipboard     *aClipboard,
              * detect mozilla use UCS2 encoding when copy-paste.
              */
             guchar *buffer = (guchar *)
-                    nsMemory::Alloc((len * sizeof(guchar)) + sizeof(PRUnichar));
+                    moz_xmalloc((len * sizeof(guchar)) + sizeof(char16_t));
             if (!buffer)
                 return;
-            PRUnichar prefix = 0xFEFF;
+            char16_t prefix = 0xFEFF;
             memcpy(buffer, &prefix, sizeof(prefix));
             memcpy(buffer + sizeof(prefix), primitive_data, len);
-            nsMemory::Free((guchar *)primitive_data);
+            free((guchar *)primitive_data);
             primitive_data = (guchar *)buffer;
             len += sizeof(prefix);
         }
@@ -620,7 +634,7 @@ nsClipboard::SelectionGetEvent(GtkClipboard     *aClipboard,
         gtk_selection_data_set(aSelectionData, selectionTarget,
                                8, /* 8 bits in a unit */
                                (const guchar *)primitive_data, len);
-        nsMemory::Free(primitive_data);
+        free(primitive_data);
     }
 
     g_free(target_name);
@@ -681,18 +695,18 @@ clipboard_clear_cb(GtkClipboard *aGtkClipboard,
  * bodyLength: pass to Mozilla
  */
 void ConvertHTMLtoUCS2(guchar * data, int32_t dataLength,
-                       PRUnichar** unicodeData, int32_t& outUnicodeLen)
+                       char16_t** unicodeData, int32_t& outUnicodeLen)
 {
     nsAutoCString charset;
     GetHTMLCharset(data, dataLength, charset);// get charset of HTML
     if (charset.EqualsLiteral("UTF-16")) {//current mozilla
         outUnicodeLen = (dataLength / 2) - 1;
-        *unicodeData = reinterpret_cast<PRUnichar*>
-                                       (nsMemory::Alloc((outUnicodeLen + sizeof('\0')) *
-                       sizeof(PRUnichar)));
+        *unicodeData = reinterpret_cast<char16_t*>
+                                       (moz_xmalloc((outUnicodeLen + sizeof('\0')) *
+                       sizeof(char16_t)));
         if (*unicodeData) {
-            memcpy(*unicodeData, data + sizeof(PRUnichar),
-                   outUnicodeLen * sizeof(PRUnichar));
+            memcpy(*unicodeData, data + sizeof(char16_t),
+                   outUnicodeLen * sizeof(char16_t));
             (*unicodeData)[outUnicodeLen] = '\0';
         }
     } else if (charset.EqualsLiteral("UNKNOWN")) {
@@ -701,32 +715,30 @@ void ConvertHTMLtoUCS2(guchar * data, int32_t dataLength,
     } else {
         // app which use "text/html" to copy&paste
         nsCOMPtr<nsIUnicodeDecoder> decoder;
-        nsresult rv;
         // get the decoder
-        nsCOMPtr<nsICharsetConverterManager> ccm =
-            do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
-        if (NS_FAILED(rv)) {
-#ifdef DEBUG_CLIPBOARD
-            g_print("        can't get CHARSET CONVERTER MANAGER service\n");
-#endif
-            outUnicodeLen = 0;
-            return;
-        }
-        rv = ccm->GetUnicodeDecoder(charset.get(), getter_AddRefs(decoder));
-        if (NS_FAILED(rv)) {
+        nsAutoCString encoding;
+        if (!EncodingUtils::FindEncodingForLabelNoReplacement(charset,
+                                                              encoding)) {
 #ifdef DEBUG_CLIPBOARD
             g_print("        get unicode decoder error\n");
 #endif
             outUnicodeLen = 0;
             return;
         }
+        decoder = EncodingUtils::DecoderForEncoding(encoding);
         // converting
-        decoder->GetMaxLength((const char *)data, dataLength, &outUnicodeLen);
+        nsresult rv = decoder->GetMaxLength((const char *)data, dataLength,
+                                            &outUnicodeLen);
+        if (NS_WARN_IF(NS_FAILED(rv))) {
+          outUnicodeLen = 0;
+          return;
+        }
+
         // |outUnicodeLen| is number of chars
         if (outUnicodeLen) {
-            *unicodeData = reinterpret_cast<PRUnichar*>
-                                           (nsMemory::Alloc((outUnicodeLen + sizeof('\0')) *
-                           sizeof(PRUnichar)));
+            *unicodeData = reinterpret_cast<char16_t*>
+                                           (moz_xmalloc((outUnicodeLen + sizeof('\0')) *
+                           sizeof(char16_t)));
             if (*unicodeData) {
                 int32_t numberTmp = dataLength;
                 decoder->Convert((const char *)data, &numberTmp,
@@ -752,7 +764,7 @@ void ConvertHTMLtoUCS2(guchar * data, int32_t dataLength,
 void GetHTMLCharset(guchar * data, int32_t dataLength, nsCString& str)
 {
     // if detect "FFFE" or "FEFF", assume UTF-16
-    PRUnichar* beginChar =  (PRUnichar*)data;
+    char16_t* beginChar =  (char16_t*)data;
     if ((beginChar[0] == 0xFFFE) || (beginChar[0] == 0xFEFF)) {
         str.AssignLiteral("UTF-16");
         return;
@@ -869,16 +881,18 @@ static GtkSelectionData* CopyRetrievedData(GtkSelectionData *aData)
         gtk_selection_data_copy(aData) : nullptr;
 }
 
-class RetrievalContext : public RefCounted<RetrievalContext> {
-public:
-    enum State { INITIAL, COMPLETED, TIMED_OUT };
-
-    RetrievalContext() : mState(INITIAL), mData(nullptr) {}
+class RetrievalContext {
     ~RetrievalContext()
     {
         MOZ_ASSERT(!mData, "Wait() wasn't called");
     }
 
+public:
+    NS_INLINE_DECL_REFCOUNTING(RetrievalContext)
+    enum State { INITIAL, COMPLETED, TIMED_OUT };
+
+    RetrievalContext() : mState(INITIAL), mData(nullptr) {}
+
     /**
      * Call this when data has been retrieved.
      */
@@ -914,52 +928,54 @@ RetrievalContext::Wait()
         return data;
     }
 
-    Display *xDisplay = GDK_DISPLAY_XDISPLAY(gdk_display_get_default()) ;
-    checkEventContext context;
-    context.cbWidget = nullptr;
-    context.selAtom = gdk_x11_atom_to_xatom(gdk_atom_intern("GDK_SELECTION",
-                                                            FALSE));
+    GdkDisplay *gdkDisplay = gdk_display_get_default();
+    if (GDK_IS_X11_DISPLAY(gdkDisplay)) {
+        Display *xDisplay = GDK_DISPLAY_XDISPLAY(gdkDisplay);
+        checkEventContext context;
+        context.cbWidget = nullptr;
+        context.selAtom = gdk_x11_atom_to_xatom(gdk_atom_intern("GDK_SELECTION",
+                                                                FALSE));
 
-    // Send X events which are relevant to the ongoing selection retrieval
-    // to the clipboard widget.  Wait until either the operation completes, or
-    // we hit our timeout.  All other X events remain queued.
+        // Send X events which are relevant to the ongoing selection retrieval
+        // to the clipboard widget.  Wait until either the operation completes, or
+        // we hit our timeout.  All other X events remain queued.
 
-    int select_result;
+        int select_result;
 
-    int cnumber = ConnectionNumber(xDisplay);
-    fd_set select_set;
-    FD_ZERO(&select_set);
-    FD_SET(cnumber, &select_set);
-    ++cnumber;
-    TimeStamp start = TimeStamp::Now();
+        int cnumber = ConnectionNumber(xDisplay);
+        fd_set select_set;
+        FD_ZERO(&select_set);
+        FD_SET(cnumber, &select_set);
+        ++cnumber;
+        TimeStamp start = TimeStamp::Now();
 
-    do {
-        XEvent xevent;
+        do {
+            XEvent xevent;
 
-        while (XCheckIfEvent(xDisplay, &xevent, checkEventProc,
-                             (XPointer) &context)) {
+            while (XCheckIfEvent(xDisplay, &xevent, checkEventProc,
+                                 (XPointer) &context)) {
 
-            if (xevent.xany.type == SelectionNotify)
-                DispatchSelectionNotifyEvent(context.cbWidget, &xevent);
-            else
-                DispatchPropertyNotifyEvent(context.cbWidget, &xevent);
+                if (xevent.xany.type == SelectionNotify)
+                    DispatchSelectionNotifyEvent(context.cbWidget, &xevent);
+                else
+                    DispatchPropertyNotifyEvent(context.cbWidget, &xevent);
 
-            if (mState == COMPLETED) {
-                void *data = mData;
-                mData = nullptr;
-                return data;
+                if (mState == COMPLETED) {
+                    void *data = mData;
+                    mData = nullptr;
+                    return data;
+                }
             }
-        }
-
-        TimeStamp now = TimeStamp::Now();
-        struct timeval tv;
-        tv.tv_sec = 0;
-        tv.tv_usec = std::max<int32_t>(0,
-            kClipboardTimeout - (now - start).ToMicroseconds());
-        select_result = select(cnumber, &select_set, nullptr, nullptr, &tv);
-    } while (select_result == 1 ||
-             (select_result == -1 && errno == EINTR));
 
+            TimeStamp now = TimeStamp::Now();
+            struct timeval tv;
+            tv.tv_sec = 0;
+            tv.tv_usec = std::max<int32_t>(0,
+                kClipboardTimeout - (now - start).ToMicroseconds());
+            select_result = select(cnumber, &select_set, nullptr, nullptr, &tv);
+        } while (select_result == 1 ||
+                 (select_result == -1 && errno == EINTR));
+    }
 #ifdef DEBUG_CLIPBOARD
     printf("exceeded clipboard timeout\n");
 #endif
@@ -982,7 +998,7 @@ wait_for_contents(GtkClipboard *clipboard, GdkAtom target)
 {
     RefPtr<RetrievalContext> context = new RetrievalContext();
     // Balanced by Release in clipboard_contents_received
-    context->AddRef();
+    context.get()->AddRef();
     gtk_clipboard_request_contents(clipboard, target,
                                    clipboard_contents_received,
                                    context.get());
@@ -1004,7 +1020,30 @@ wait_for_text(GtkClipboard *clipboard)
 {
     RefPtr<RetrievalContext> context = new RetrievalContext();
     // Balanced by Release in clipboard_text_received
-    context->AddRef();
+    context.get()->AddRef();
     gtk_clipboard_request_text(clipboard, clipboard_text_received, context.get());
     return static_cast<gchar*>(context->Wait());
 }
+
+static GdkFilterReturn
+selection_request_filter(GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
+{
+    XEvent *xevent = static_cast<XEvent*>(gdk_xevent);
+    if (xevent->xany.type == SelectionRequest) {
+        if (xevent->xselectionrequest.requestor == None)
+            return GDK_FILTER_REMOVE;
+
+        GdkDisplay *display = gdk_x11_lookup_xdisplay(
+                xevent->xselectionrequest.display);
+        if (!display)
+            return GDK_FILTER_REMOVE;
+
+        GdkWindow *window = gdk_x11_window_foreign_new_for_display(display,
+                xevent->xselectionrequest.requestor);
+        if (!window)
+            return GDK_FILTER_REMOVE;
+
+        g_object_unref(window);
+    }
+    return GDK_FILTER_CONTINUE;
+}
