@@ -29,6 +29,7 @@
 #ifdef MOZ_X11
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
+#include "gfxXlibSurface.h"
 #endif //MOZ_X11
 
 #include "nsXULAppAPI.h"
@@ -39,7 +40,6 @@
 #include "mozqwidget.h"
 
 #include "nsIdleService.h"
-#include "nsRenderingContext.h"
 #include "nsIRollupListener.h"
 #include "nsWidgetsCID.h"
 #include "nsQtKeyUtils.h"
@@ -60,8 +60,6 @@
 #include "nsAutoPtr.h"
 
 #include "gfxQtPlatform.h"
-#include "gfxContext.h"
-#include "gfxImageSurface.h"
 
 #include "nsIDOMWheelEvent.h"
 
@@ -71,7 +69,6 @@
 #include "keysym2ucs.h"
 #endif
 
-#include "gfxUtils.h"
 #include "Layers.h"
 #include "GLContextProvider.h"
 
@@ -91,8 +88,8 @@ static bool sAltGrModifier = false;
 
 static void find_first_visible_parent(QWindow* aItem, QWindow*& aVisibleItem);
 static bool is_mouse_in_window (MozQWidget* aWindow, double aMouseX, double aMouseY);
-static bool isContextMenuKeyEvent(const QKeyEvent *qe);
-static void InitKeyEvent(WidgetKeyboardEvent &aEvent, QKeyEvent *aQEvent);
+
+NS_IMPL_ISUPPORTS_INHERITED0(nsWindow, nsBaseWidget)
 
 nsWindow::nsWindow()
 {
@@ -136,18 +133,17 @@ nsWindow::~nsWindow()
 }
 
 nsresult
-nsWindow::Create(nsIWidget        *aParent,
-                 nsNativeWidget    aNativeParent,
-                 const nsIntRect  &aRect,
-                 nsDeviceContext *aContext,
-                 nsWidgetInitData *aInitData)
+nsWindow::Create(nsIWidget* aParent,
+                 nsNativeWidget aNativeParent,
+                 const LayoutDeviceIntRect& aRect,
+                 nsWidgetInitData* aInitData)
 {
     // only set the base parent if we're not going to be a dialog or a
     // toplevel
     nsIWidget *baseParent = aParent;
 
     // initialize all the common bits of this class
-    BaseCreate(baseParent, aRect, aContext, aInitData);
+    BaseCreate(baseParent, aRect, aInitData);
 
     mVisible = true;
 
@@ -155,7 +151,7 @@ nsWindow::Create(nsIWidget        *aParent,
     mParent = (nsWindow *)aParent;
 
     // save our bounds
-    mBounds = aRect;
+    mBounds = aRect.ToUnknownRect();
 
     // find native parent
     MozQWidget *parent = nullptr;
@@ -227,7 +223,11 @@ nsWindow::createQWidget(MozQWidget* parent,
 
     widget->setObjectName(QString(windowName));
     if (mWindowType == eWindowType_invisible) {
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 1, 0))
         widget->setVisibility(QWindow::Hidden);
+#else
+        widget->hide();
+#endif
     }
     if (mWindowType == eWindowType_dialog) {
         widget->setModality(Qt::WindowModal);
@@ -270,7 +270,7 @@ nsWindow::Destroy(void)
     if (rollupListener) {
         nsCOMPtr<nsIWidget> rollupWidget = rollupListener->GetRollupWidget();
         if (static_cast<nsIWidget *>(this) == rollupWidget) {
-            rollupListener->Rollup(0, nullptr, nullptr);
+            rollupListener->Rollup(0, false, nullptr, nullptr);
         }
     }
 
@@ -466,7 +466,8 @@ nsWindow::Resize(double aWidth, double aHeight, bool aRepaint)
     // synthesize a resize event if this isn't a toplevel
     if (mIsTopLevel || mListenForResizes) {
         nsEventStatus status;
-        DispatchResizeEvent(mBounds, status);
+        DispatchResizeEvent(LayoutDeviceIntRect::FromUnknownRect(mBounds),
+                            status);
     }
 
     NotifyRollupGeometryChange();
@@ -529,7 +530,8 @@ nsWindow::Resize(double aX, double aY, double aWidth, double aHeight,
     if (mIsTopLevel || mListenForResizes) {
         // synthesize a resize event
         nsEventStatus status;
-        DispatchResizeEvent(mBounds, status);
+        DispatchResizeEvent(LayoutDeviceIntRect::FromUnknownRect(mBounds),
+                            status);
     }
 
     if (aRepaint) {
@@ -601,11 +603,13 @@ nsWindow::ConfigureChildren(const nsTArray<nsIWidget::Configuration>& aConfigura
         NS_ASSERTION(w->GetParent() == this,
                      "Configured widget is not a child");
 
-        if (w->mBounds.Size() != configuration.mBounds.Size()) {
+        LayoutDeviceIntRect wBounds =
+            LayoutDeviceIntRect::FromUnknownRect(w->mBounds);
+        if (wBounds.Size() != configuration.mBounds.Size()) {
             w->Resize(configuration.mBounds.x, configuration.mBounds.y,
                       configuration.mBounds.width, configuration.mBounds.height,
                       true);
-        } else if (w->mBounds.TopLeft() != configuration.mBounds.TopLeft()) {
+        } else if (wBounds.TopLeft() != configuration.mBounds.TopLeft()) {
             w->Move(configuration.mBounds.x, configuration.mBounds.y);
         }
     }
@@ -613,10 +617,10 @@ nsWindow::ConfigureChildren(const nsTArray<nsIWidget::Configuration>& aConfigura
 }
 
 NS_IMETHODIMP
-nsWindow::Invalidate(const nsIntRect &aRect)
+nsWindow::Invalidate(const LayoutDeviceIntRect& aRect)
 {
     LOGDRAW(("Invalidate (rect) [%p,%p]: %d %d %d %d\n", (void *)this,
-             (void*)mWidget,aRect.x, aRect.y, aRect.width, aRect.height));
+             (void*)mWidget, aRect.x, aRect.y, aRect.width, aRect.height));
 
     if (!mWidget) {
         return NS_OK;
@@ -627,15 +631,15 @@ nsWindow::Invalidate(const nsIntRect &aRect)
     return NS_OK;
 }
 
-nsIntPoint
+LayoutDeviceIntPoint
 nsWindow::WidgetToScreenOffset()
 {
-    NS_ENSURE_TRUE(mWidget, nsIntPoint(0,0));
+    NS_ENSURE_TRUE(mWidget, LayoutDeviceIntPoint(0,0));
 
     QPoint origin(0, 0);
     origin = mWidget->mapToGlobal(origin);
 
-    return nsIntPoint(origin.x(), origin.y());
+    return LayoutDeviceIntPoint(origin.x(), origin.y());
 }
 
 void*
@@ -660,6 +664,10 @@ nsWindow::GetNativeData(uint32_t aDataType)
     case NS_NATIVE_SHELLWIDGET: {
         break;
     }
+    case NS_RAW_NATIVE_IME_CONTEXT:
+        // Our qt widget looks like using only one context per process.
+        // However, it's better to set the context's pointer.
+        return qApp->inputMethod();
     default:
         NS_WARNING("nsWindow::GetNativeData called with bad value");
         return nullptr;
@@ -712,10 +720,6 @@ NS_IMETHODIMP_(InputContext)
 nsWindow::GetInputContext()
 {
     mInputContext.mIMEState.mOpen = IMEState::OPEN_STATE_NOT_SUPPORTED;
-    // Our qt widget looks like using only one context per process.
-    // However, it's better to set the context's pointer.
-    mInputContext.mNativeIMEContext = qApp->inputMethod();
-
     return mInputContext;
 }
 
@@ -733,7 +737,7 @@ nsWindow::ReparentNativeWidget(nsIWidget *aNewParent)
 }
 
 NS_IMETHODIMP
-nsWindow::MakeFullScreen(bool aFullScreen)
+nsWindow::MakeFullScreen(bool aFullScreen, nsIScreen*)
 {
     NS_ENSURE_TRUE(mWidget, NS_ERROR_FAILURE);
 
@@ -805,13 +809,39 @@ nsWindow::GetGLFrameBufferFormat()
     return LOCAL_GL_NONE;
 }
 
+already_AddRefed<DrawTarget>
+nsWindow::StartRemoteDrawing()
+{
+    if (!mWidget) {
+        return nullptr;
+    }
+
+#ifdef MOZ_X11
+    Display* dpy = gfxQtPlatform::GetXDisplay(mWidget);
+    Screen* screen = DefaultScreenOfDisplay(dpy);
+    Visual* defaultVisual = DefaultVisualOfScreen(screen);
+    gfxASurface* surf = new gfxXlibSurface(dpy, mWidget->winId(), defaultVisual,
+                                           IntSize(mWidget->width(),
+                                                   mWidget->height()));
+
+    IntSize size(surf->GetSize().width, surf->GetSize().height);
+    if (size.width <= 0 || size.height <= 0) {
+        return nullptr;
+    }
+
+    return gfxPlatform::GetPlatform()->CreateDrawTargetForSurface(surf, size);
+#else
+    return nullptr;
+#endif
+}
+
 NS_IMETHODIMP
 nsWindow::SetCursor(nsCursor aCursor)
 {
-    if (mCursor == aCursor) {
+    if (mCursor == aCursor && !mUpdateCursor) {
         return NS_OK;
     }
-
+    mUpdateCursor = false;
     mCursor = aCursor;
     if (mWidget) {
         mWidget->SetCursor(mCursor);
@@ -845,7 +875,8 @@ nsWindow::OnPaint()
 
     switch (GetLayerManager()->GetBackendType()) {
         case mozilla::layers::LayersBackend::LAYERS_CLIENT: {
-            nsIntRegion region(nsIntRect(0, 0, mWidget->width(), mWidget->height()));
+            LayoutDeviceIntRegion region(
+              LayoutDeviceIntRect(0, 0, mWidget->width(), mWidget->height()));
             listener->PaintWindow(this, region);
             break;
         }
@@ -879,7 +910,7 @@ nsWindow::moveEvent(QMoveEvent* aEvent)
 nsEventStatus
 nsWindow::resizeEvent(QResizeEvent* aEvent)
 {
-    nsIntRect rect;
+    LayoutDeviceIntRect rect;
 
     // Generate XPFE resize event
     GetBounds(rect);
@@ -908,6 +939,47 @@ nsWindow::mouseMoveEvent(QMouseEvent* aEvent)
     return nsEventStatus_eIgnore;
 }
 
+static void
+InitMouseEvent(WidgetMouseEvent& aMouseEvent, QMouseEvent* aEvent,
+               int aClickCount)
+{
+    aMouseEvent.refPoint.x = nscoord(aEvent->pos().x());
+    aMouseEvent.refPoint.y = nscoord(aEvent->pos().y());
+
+    aMouseEvent.InitBasicModifiers(aEvent->modifiers() & Qt::ControlModifier,
+                                   aEvent->modifiers() & Qt::AltModifier,
+                                   aEvent->modifiers() & Qt::ShiftModifier,
+                                   aEvent->modifiers() & Qt::MetaModifier);
+    aMouseEvent.clickCount = aClickCount;
+
+    switch (aEvent->button()) {
+    case Qt::LeftButton:
+        aMouseEvent.button = WidgetMouseEvent::eLeftButton;
+        break;
+    case Qt::RightButton:
+        aMouseEvent.button = WidgetMouseEvent::eRightButton;
+        break;
+    case Qt::MiddleButton:
+        aMouseEvent.button = WidgetMouseEvent::eMiddleButton;
+        break;
+    default:
+        break;
+    }
+}
+
+static bool
+IsAcceptedButton(Qt::MouseButton button)
+{
+    switch (button) {
+    case Qt::LeftButton:
+    case Qt::RightButton:
+    case Qt::MiddleButton:
+        return true;
+    default:
+        return false;
+    }
+}
+
 nsEventStatus
 nsWindow::mousePressEvent(QMouseEvent* aEvent)
 {
@@ -921,37 +993,27 @@ nsWindow::mousePressEvent(QMouseEvent* aEvent)
     if (mWidget)
         pos = mWidget->mapToGlobal(pos);
 
-    if (CheckForRollup( pos.x(), pos.y(), false))
+    if (CheckForRollup(pos.x(), pos.y(), false))
         return nsEventStatus_eIgnore;
 
-    uint16_t      domButton;
-    switch (aEvent->button()) {
-    case Qt::MidButton:
-        domButton = WidgetMouseEvent::eMiddleButton;
-        break;
-    case Qt::RightButton:
-        domButton = WidgetMouseEvent::eRightButton;
-        break;
-    default:
-        domButton = WidgetMouseEvent::eLeftButton;
-        break;
+    if (!IsAcceptedButton(aEvent->button())) {
+        if (aEvent->button() == Qt::BackButton)
+            return DispatchCommandEvent(nsGkAtoms::Back);
+        if (aEvent->button() == Qt::ForwardButton)
+            return DispatchCommandEvent(nsGkAtoms::Forward);
+        return nsEventStatus_eIgnore;
     }
 
-    WidgetMouseEvent event(true, NS_MOUSE_BUTTON_DOWN, this,
-                           WidgetMouseEvent::eReal);
-    event.button = domButton;
-    InitButtonEvent(event, aEvent, 1);
-
-    LOG(("%s [%p] button: %d\n", __PRETTY_FUNCTION__, (void*)this, domButton));
-
+    WidgetMouseEvent event(true, eMouseDown, this, WidgetMouseEvent::eReal);
+    InitMouseEvent(event, aEvent, 1);
     nsEventStatus status = DispatchEvent(&event);
 
-    // right menu click on linux should also pop up a context menu
-    if (domButton == WidgetMouseEvent::eRightButton &&
+    // Right click on linux should also pop up a context menu.
+    if (event.button == WidgetMouseEvent::eRightButton &&
         MOZ_LIKELY(!mIsDestroyed)) {
-        WidgetMouseEvent contextMenuEvent(true, NS_CONTEXTMENU, this,
+        WidgetMouseEvent contextMenuEvent(true, eContextMenu, this,
                                           WidgetMouseEvent::eReal);
-        InitButtonEvent(contextMenuEvent, aEvent, 1);
+        InitMouseEvent(contextMenuEvent, aEvent, 1);
         DispatchEvent(&contextMenuEvent, status);
     }
 
@@ -964,55 +1026,26 @@ nsWindow::mouseReleaseEvent(QMouseEvent* aEvent)
     // The user has done something.
     UserActivity();
 
-    uint16_t domButton;
+    if (!IsAcceptedButton(aEvent->button()))
+        return nsEventStatus_eIgnore;
 
-    switch (aEvent->button()) {
-    case Qt::MidButton:
-        domButton = WidgetMouseEvent::eMiddleButton;
-        break;
-    case Qt::RightButton:
-        domButton = WidgetMouseEvent::eRightButton;
-        break;
-    default:
-        domButton = WidgetMouseEvent::eLeftButton;
-        break;
-    }
-
-    LOG(("%s [%p] button: %d\n", __PRETTY_FUNCTION__, (void*)this, domButton));
-
-    WidgetMouseEvent event(true, NS_MOUSE_BUTTON_UP, this,
-                           WidgetMouseEvent::eReal);
-    event.button = domButton;
-    InitButtonEvent(event, aEvent, 1);
-
-    nsEventStatus status = DispatchEvent(&event);
-
-    return status;
+    WidgetMouseEvent event(true, eMouseUp, this, WidgetMouseEvent::eReal);
+    InitMouseEvent(event, aEvent, 1);
+    return DispatchEvent(&event);
 }
 
 nsEventStatus
 nsWindow::mouseDoubleClickEvent(QMouseEvent* aEvent)
 {
-    uint32_t eventType;
+    // The user has done something.
+    UserActivity();
 
-    switch (aEvent->button()) {
-    case Qt::MidButton:
-        eventType = WidgetMouseEvent::eMiddleButton;
-        break;
-    case Qt::RightButton:
-        eventType = WidgetMouseEvent::eRightButton;
-        break;
-    default:
-        eventType = WidgetMouseEvent::eLeftButton;
-        break;
-    }
+    if (!IsAcceptedButton(aEvent->button()))
+        return nsEventStatus_eIgnore;
 
-    WidgetMouseEvent event(true, NS_MOUSE_DOUBLECLICK, this,
+    WidgetMouseEvent event(true, eMouseDoubleClick, this,
                            WidgetMouseEvent::eReal);
-    event.button = eventType;
-
-    InitButtonEvent(event, aEvent, 2);
-    //pressed
+    InitMouseEvent(event, aEvent, 2);
     return DispatchEvent(&event);
 }
 
@@ -1046,6 +1079,57 @@ nsWindow::focusOutEvent(QFocusEvent* aEvent)
     return nsEventStatus_eIgnore;
 }
 
+static bool
+IsContextMenuKeyEvent(const QKeyEvent* aQEvent)
+{
+    if (aQEvent->modifiers() & (Qt::ControlModifier |
+                                Qt::AltModifier |
+                                Qt::MetaModifier)) {
+        return false;
+    }
+
+    bool isShift = aQEvent->modifiers() & Qt::ShiftModifier;
+    uint32_t keyCode = QtKeyCodeToDOMKeyCode(aQEvent->key());
+    return (keyCode == NS_VK_F10 && isShift) ||
+           (keyCode == NS_VK_CONTEXT_MENU && !isShift);
+}
+
+static void
+InitKeyEvent(WidgetKeyboardEvent& aEvent, QKeyEvent* aQEvent)
+{
+    aEvent.InitBasicModifiers(aQEvent->modifiers() & Qt::ControlModifier,
+                              aQEvent->modifiers() & Qt::AltModifier,
+                              aQEvent->modifiers() & Qt::ShiftModifier,
+                              aQEvent->modifiers() & Qt::MetaModifier);
+
+    aEvent.mIsRepeat =
+        (aEvent.mMessage == eKeyDown || aEvent.mMessage == eKeyPress) &&
+        aQEvent->isAutoRepeat();
+    aEvent.time = 0;
+
+    if (sAltGrModifier) {
+        aEvent.modifiers |= (MODIFIER_CONTROL | MODIFIER_ALT);
+    }
+
+    if (aQEvent->text().length() && aQEvent->text()[0].isPrint()) {
+        aEvent.charCode = (int32_t) aQEvent->text()[0].unicode();
+        aEvent.keyCode = 0;
+        aEvent.mKeyNameIndex = KEY_NAME_INDEX_PrintableKey;
+    } else {
+        aEvent.charCode = 0;
+        aEvent.keyCode = QtKeyCodeToDOMKeyCode(aQEvent->key());
+        aEvent.mKeyNameIndex = QtKeyCodeToDOMKeyNameIndex(aQEvent->key());
+    }
+
+    aEvent.mCodeNameIndex = ScanCodeToDOMCodeNameIndex(aQEvent->nativeScanCode());
+
+    // The transformations above and in qt for the keyval are not invertible
+    // so link to the QKeyEvent (which will vanish soon after return from the
+    // event callback) to give plugins access to hardware_keycode and state.
+    // (An XEvent would be nice but the QKeyEvent is good enough.)
+    aEvent.mPluginEvent.Copy(*aQEvent);
+}
+
 nsEventStatus
 nsWindow::keyPressEvent(QKeyEvent* aEvent)
 {
@@ -1058,102 +1142,26 @@ nsWindow::keyPressEvent(QKeyEvent* aEvent)
         sAltGrModifier = true;
     }
 
-#ifdef MOZ_X11
-    // before we dispatch a key, check if it's the context menu key.
+    // Before we dispatch a key, check if it's the context menu key.
     // If so, send a context menu key event instead.
-    if (isContextMenuKeyEvent(aEvent)) {
-        WidgetMouseEvent contextMenuEvent(true, NS_CONTEXTMENU, this,
+    if (IsContextMenuKeyEvent(aEvent)) {
+        WidgetMouseEvent contextMenuEvent(true, eContextMenu, this,
                                           WidgetMouseEvent::eReal,
                                           WidgetMouseEvent::eContextMenuKey);
-        //keyEventToContextMenuEvent(&event, &contextMenuEvent);
         return DispatchEvent(&contextMenuEvent);
     }
 
-    uint32_t domCharCode = 0;
+    //:TODO: fix shortcuts hebrew for non X11,
+    //see Bug 562195##51
+
     uint32_t domKeyCode = QtKeyCodeToDOMKeyCode(aEvent->key());
 
-    // get keymap and modifier map from the Xserver
-    Display *display = gfxQtPlatform::GetXDisplay(mWidget);
-    int x_min_keycode = 0, x_max_keycode = 0, xkeysyms_per_keycode;
-    XDisplayKeycodes(display, &x_min_keycode, &x_max_keycode);
-    XModifierKeymap *xmodmap = XGetModifierMapping(display);
-    if (!xmodmap)
-        return nsEventStatus_eIgnore;
-
-    KeySym *xkeymap = XGetKeyboardMapping(display, x_min_keycode, x_max_keycode - x_min_keycode,
-                                          &xkeysyms_per_keycode);
-    if (!xkeymap) {
-        XFreeModifiermap(xmodmap);
-        return nsEventStatus_eIgnore;
-    }
-
-    // create modifier masks
-    qint32 shift_mask = 0, shift_lock_mask = 0, caps_lock_mask = 0, num_lock_mask = 0;
-
-    for (int i = 0; i < 8 * xmodmap->max_keypermod; ++i) {
-        qint32 maskbit = 1 << (i / xmodmap->max_keypermod);
-        KeyCode modkeycode = xmodmap->modifiermap[i];
-        if (modkeycode == NoSymbol) {
-            continue;
-        }
-
-        quint32 mapindex = (modkeycode - x_min_keycode) * xkeysyms_per_keycode;
-        for (int j = 0; j < xkeysyms_per_keycode; ++j) {
-            KeySym modkeysym = xkeymap[mapindex + j];
-            switch (modkeysym) {
-                case XK_Num_Lock:
-                    num_lock_mask |= maskbit;
-                    break;
-                case XK_Caps_Lock:
-                    caps_lock_mask |= maskbit;
-                    break;
-                case XK_Shift_Lock:
-                    shift_lock_mask |= maskbit;
-                    break;
-                case XK_Shift_L:
-                case XK_Shift_R:
-                    shift_mask |= maskbit;
-                    break;
-            }
-        }
-    }
-    // indicate whether is down or not
-    bool shift_state = ((shift_mask & aEvent->nativeModifiers()) != 0) ^
-                          (bool)(shift_lock_mask & aEvent->nativeModifiers());
-    bool capslock_state = (bool)(caps_lock_mask & aEvent->nativeModifiers());
-
-    // try to find a keysym that we can translate to a DOMKeyCode
-    // this is needed because some of Qt's keycodes cannot be translated
-    // TODO: use US keyboard keymap instead of localised keymap
-    if (!domKeyCode &&
-        aEvent->nativeScanCode() >= (quint32)x_min_keycode &&
-        aEvent->nativeScanCode() <= (quint32)x_max_keycode) {
-        int index = (aEvent->nativeScanCode() - x_min_keycode) * xkeysyms_per_keycode;
-        for(int i = 0; (i < xkeysyms_per_keycode) && (domKeyCode == (quint32)NoSymbol); ++i) {
-            domKeyCode = QtKeyCodeToDOMKeyCode(xkeymap[index + i]);
-        }
-    }
-
-    // store character in domCharCode
-    if (aEvent->text().length() && aEvent->text()[0].isPrint())
-        domCharCode = (int32_t) aEvent->text()[0].unicode();
-
-    KeyNameIndex keyNameIndex =
-        domCharCode ? KEY_NAME_INDEX_PrintableKey :
-                      QtKeyCodeToDOMKeyNameIndex(aEvent->key());
-
-    // If the key isn't autorepeat, we need to send the initial down event
     if (!aEvent->isAutoRepeat() && !IsKeyDown(domKeyCode)) {
-        // send the key down event
-
         SetKeyDownFlag(domKeyCode);
 
-        WidgetKeyboardEvent downEvent(true, NS_KEY_DOWN, this);
+        WidgetKeyboardEvent downEvent(true, eKeyDown, this);
         InitKeyEvent(downEvent, aEvent);
 
-        downEvent.keyCode = domKeyCode;
-        downEvent.mKeyNameIndex = keyNameIndex;
-
         nsEventStatus status = DispatchEvent(&downEvent);
 
         // DispatchEvent can Destroy us (bug 378273)
@@ -1168,8 +1176,8 @@ nsWindow::keyPressEvent(QKeyEvent* aEvent)
         }
     }
 
-    // Don't pass modifiers as NS_KEY_PRESS events.
-    // Instead of selectively excluding some keys from NS_KEY_PRESS events,
+    // Don't pass modifiers as eKeyPress events.
+    // Instead of selectively excluding some keys from eKeyPress events,
     // we instead selectively include (as per MSDN spec
     // ( http://msdn.microsoft.com/en-us/library/system.windows.forms.control.keypress%28VS.71%29.aspx );
     // no official spec covers KeyPress events).
@@ -1178,7 +1186,6 @@ nsWindow::keyPressEvent(QKeyEvent* aEvent)
         aEvent->key() == Qt::Key_Meta    ||
         aEvent->key() == Qt::Key_Alt     ||
         aEvent->key() == Qt::Key_AltGr) {
-
         return nsEventStatus_eIgnore;
     }
 
@@ -1200,222 +1207,30 @@ nsWindow::keyPressEvent(QKeyEvent* aEvent)
             return DispatchCommandEvent(nsGkAtoms::Home);
         case Qt::Key_Copy:
         case Qt::Key_F16: // F16, F20, F18, F14 are old keysyms for Copy Cut Paste Undo
-            return DispatchContentCommandEvent(NS_CONTENT_COMMAND_COPY);
+            return DispatchContentCommandEvent(eContentCommandCopy);
         case Qt::Key_Cut:
         case Qt::Key_F20:
-            return DispatchContentCommandEvent(NS_CONTENT_COMMAND_CUT);
+            return DispatchContentCommandEvent(eContentCommandCut);
         case Qt::Key_Paste:
         case Qt::Key_F18:
         case Qt::Key_F9:
-            return DispatchContentCommandEvent(NS_CONTENT_COMMAND_PASTE);
+            return DispatchContentCommandEvent(eContentCommandPaste);
         case Qt::Key_F14:
-            return DispatchContentCommandEvent(NS_CONTENT_COMMAND_UNDO);
+            return DispatchContentCommandEvent(eContentCommandUndo);
     }
 
     // Qt::Key_Redo and Qt::Key_Undo are not available yet.
     if (aEvent->nativeVirtualKey() == 0xff66) {
-        return DispatchContentCommandEvent(NS_CONTENT_COMMAND_REDO);
+        return DispatchContentCommandEvent(eContentCommandRedo);
     }
     if (aEvent->nativeVirtualKey() == 0xff65) {
-        return DispatchContentCommandEvent(NS_CONTENT_COMMAND_UNDO);
+        return DispatchContentCommandEvent(eContentCommandUndo);
     }
 
-    WidgetKeyboardEvent event(true, NS_KEY_PRESS, this);
+    WidgetKeyboardEvent event(true, eKeyPress, this);
     InitKeyEvent(event, aEvent);
-
-    // If there is no charcode attainable from the text, try to
-    // generate it from the keycode. Check shift state for case
-    // Also replace the charcode if ControlModifier is the only
-    // pressed Modifier
-    if ((!domCharCode) &&
-        (QGuiApplication::keyboardModifiers() &
-        (Qt::ControlModifier | Qt::AltModifier | Qt::MetaModifier))) {
-
-        // get a character from X11 key map
-        KeySym keysym = aEvent->nativeVirtualKey();
-        if (keysym) {
-            domCharCode = (uint32_t) keysym2ucs(keysym);
-            if (domCharCode == -1 || !QChar((quint32)domCharCode).isPrint()) {
-                domCharCode = 0;
-            }
-        }
-
-        // if Ctrl is pressed and domCharCode is not a ASCII character
-        if (domCharCode > 0xFF && (QGuiApplication::keyboardModifiers() & Qt::ControlModifier)) {
-            // replace Unicode character
-            int index = (aEvent->nativeScanCode() - x_min_keycode) * xkeysyms_per_keycode;
-            for (int i = 0; i < xkeysyms_per_keycode; ++i) {
-                if (xkeymap[index + i] <= 0xFF && !shift_state) {
-                    domCharCode = (uint32_t) QChar::toLower((uint) xkeymap[index + i]);
-                    break;
-                }
-            }
-        }
-
-    } else { // The key event should cause a character input.
-             // At that time, we need to reset the modifiers
-             // because nsEditor will not accept a key event
-             // for text input if one or more modifiers are set.
-        event.modifiers &= ~(MODIFIER_CONTROL |
-                             MODIFIER_ALT |
-                             MODIFIER_META);
-    }
-
-    KeySym keysym = NoSymbol;
-    int index = (aEvent->nativeScanCode() - x_min_keycode) * xkeysyms_per_keycode;
-    for (int i = 0; i < xkeysyms_per_keycode; ++i) {
-        if (xkeymap[index + i] == aEvent->nativeVirtualKey()) {
-            if ((i % 2) == 0) { // shifted char
-                keysym = xkeymap[index + i + 1];
-                break;
-            } else { // unshifted char
-                keysym = xkeymap[index + i - 1];
-                break;
-            }
-        }
-        if (xkeysyms_per_keycode - 1 == i) {
-            qWarning() << "Symbol '" << aEvent->nativeVirtualKey() << "' not found";
-        }
-    }
-    QChar unshiftedChar(domCharCode);
-    long ucs = keysym2ucs(keysym);
-    ucs = ucs == -1 ? 0 : ucs;
-    QChar shiftedChar((uint)ucs);
-
-    // append alternativeCharCodes if modifier is pressed
-    // append an additional alternativeCharCodes if domCharCode is not a Latin character
-    // and if one of these modifiers is pressed (i.e. Ctrl, Alt, Meta)
-    if (domCharCode &&
-        (QGuiApplication::keyboardModifiers() &
-        (Qt::ControlModifier | Qt::AltModifier | Qt::MetaModifier))) {
-
-        event.charCode = domCharCode;
-        event.keyCode = 0;
-        AlternativeCharCode altCharCode(0, 0);
-        // if character has a lower and upper representation
-        if ((unshiftedChar.isUpper() || unshiftedChar.isLower()) &&
-            unshiftedChar.toLower() == shiftedChar.toLower()) {
-            if (shift_state ^ capslock_state) {
-                altCharCode.mUnshiftedCharCode = (uint32_t) QChar::toUpper((uint)domCharCode);
-                altCharCode.mShiftedCharCode = (uint32_t) QChar::toLower((uint)domCharCode);
-            } else {
-                altCharCode.mUnshiftedCharCode = (uint32_t) QChar::toLower((uint)domCharCode);
-                altCharCode.mShiftedCharCode = (uint32_t) QChar::toUpper((uint)domCharCode);
-            }
-        } else {
-            altCharCode.mUnshiftedCharCode = (uint32_t) unshiftedChar.unicode();
-            altCharCode.mShiftedCharCode = (uint32_t) shiftedChar.unicode();
-        }
-
-        // append alternative char code to event
-        if ((altCharCode.mUnshiftedCharCode && altCharCode.mUnshiftedCharCode != domCharCode) ||
-            (altCharCode.mShiftedCharCode && altCharCode.mShiftedCharCode != domCharCode)) {
-            event.alternativeCharCodes.AppendElement(altCharCode);
-        }
-
-        // check if the alternative char codes are latin-1
-        if (altCharCode.mUnshiftedCharCode > 0xFF || altCharCode.mShiftedCharCode > 0xFF) {
-            altCharCode.mUnshiftedCharCode = altCharCode.mShiftedCharCode = 0;
-
-            // find latin char for keycode
-            KeySym keysym = NoSymbol;
-            int index = (aEvent->nativeScanCode() - x_min_keycode) * xkeysyms_per_keycode;
-            // find first shifted and unshifted Latin-Char in XKeyMap
-            for (int i = 0; i < xkeysyms_per_keycode; ++i) {
-                keysym = xkeymap[index + i];
-                if (keysym && keysym <= 0xFF) {
-                    if ((shift_state && (i % 2 == 1)) ||
-                        (!shift_state && (i % 2 == 0))) {
-                        altCharCode.mUnshiftedCharCode = altCharCode.mUnshiftedCharCode ?
-                            altCharCode.mUnshiftedCharCode :
-                            keysym;
-                    } else {
-                        altCharCode.mShiftedCharCode = altCharCode.mShiftedCharCode ?
-                            altCharCode.mShiftedCharCode :
-                            keysym;
-                    }
-                    if (altCharCode.mUnshiftedCharCode && altCharCode.mShiftedCharCode) {
-                        break;
-                    }
-                }
-            }
-
-            if (altCharCode.mUnshiftedCharCode || altCharCode.mShiftedCharCode) {
-                event.alternativeCharCodes.AppendElement(altCharCode);
-            }
-        }
-    } else {
-        event.charCode = domCharCode;
-    }
-
-    if (xmodmap) {
-        XFreeModifiermap(xmodmap);
-    }
-    if (xkeymap) {
-        XFree(xkeymap);
-    }
-
-    event.keyCode = domCharCode ? 0 : domKeyCode;
-    event.mKeyNameIndex = keyNameIndex;
-    // send the key press event
+    // Seend the key press event
     return DispatchEvent(&event);
-#else
-
-    //:TODO: fix shortcuts hebrew for non X11,
-    //see Bug 562195##51
-
-    // before we dispatch a key, check if it's the context menu key.
-    // If so, send a context menu key event instead.
-    if (isContextMenuKeyEvent(aEvent)) {
-        WidgetMouseEvent contextMenuEvent(true, NS_CONTEXTMENU, this,
-                                          WidgetMouseEvent::eReal,
-                                          WidgetMouseEvent::eContextMenuKey);
-        //keyEventToContextMenuEvent(&event, &contextMenuEvent);
-        return DispatchEvent(&contextMenuEvent);
-    }
-
-    uint32_t domCharCode = 0;
-    uint32_t domKeyCode = QtKeyCodeToDOMKeyCode(aEvent->key());
-
-    if (aEvent->text().length() && aEvent->text()[0].isPrint()) {
-        domCharCode = (int32_t) aEvent->text()[0].unicode();
-    }
-
-    KeyNameIndex keyNameIndex =
-        domCharCode ? KEY_NAME_INDEX_PrintableKey :
-                      QtKeyCodeToDOMKeyNameIndex(aEvent->key());
-
-    // If the key isn't autorepeat, we need to send the initial down event
-    if (!aEvent->isAutoRepeat() && !IsKeyDown(domKeyCode)) {
-        // send the key down event
-
-        SetKeyDownFlag(domKeyCode);
-
-        WidgetKeyboardEvent downEvent(true, NS_KEY_DOWN, this);
-        InitKeyEvent(downEvent, aEvent);
-
-        downEvent.keyCode = domKeyCode;
-        downEvent.mKeyNameIndex = keyNameIndex;
-
-        nsEventStatus status = DispatchEvent(&downEvent);
-
-        // If prevent default on keydown, don't dispatch keypress event
-        if (status == nsEventStatus_eConsumeNoDefault) {
-            return nsEventStatus_eConsumeNoDefault;
-        }
-    }
-
-    WidgetKeyboardEvent event(true, NS_KEY_PRESS, this);
-    InitKeyEvent(event, aEvent);
-
-    event.charCode = domCharCode;
-
-    event.keyCode = domCharCode ? 0 : domKeyCode;
-    event.mKeyNameIndex = keyNameIndex;
-
-    // send the key press event
-    return DispatchEvent(&event);
-#endif
 }
 
 nsEventStatus
@@ -1426,50 +1241,19 @@ nsWindow::keyReleaseEvent(QKeyEvent* aEvent)
     // The user has done something.
     UserActivity();
 
-    if (isContextMenuKeyEvent(aEvent)) {
+    if (IsContextMenuKeyEvent(aEvent)) {
         // er, what do we do here? DoDefault or NoDefault?
         return nsEventStatus_eConsumeDoDefault;
     }
 
-    uint32_t domKeyCode = QtKeyCodeToDOMKeyCode(aEvent->key());
-
-#ifdef MOZ_X11
-    if (!domKeyCode) {
-        // get keymap from the Xserver
-        Display *display = gfxQtPlatform::GetXDisplay(mWidget);
-        int x_min_keycode = 0, x_max_keycode = 0, xkeysyms_per_keycode;
-        XDisplayKeycodes(display, &x_min_keycode, &x_max_keycode);
-        KeySym *xkeymap = XGetKeyboardMapping(display, x_min_keycode, x_max_keycode - x_min_keycode,
-                                              &xkeysyms_per_keycode);
-
-        if (aEvent->nativeScanCode() >= (quint32)x_min_keycode &&
-            aEvent->nativeScanCode() <= (quint32)x_max_keycode) {
-            int index = (aEvent->nativeScanCode() - x_min_keycode) * xkeysyms_per_keycode;
-            for(int i = 0; (i < xkeysyms_per_keycode) && (domKeyCode == (quint32)NoSymbol); ++i) {
-                domKeyCode = QtKeyCodeToDOMKeyCode(xkeymap[index + i]);
-            }
-        }
-
-        if (xkeymap) {
-            XFree(xkeymap);
-        }
-    }
-#endif // MOZ_X11
-
     // send the key event as a key up event
-    WidgetKeyboardEvent event(true, NS_KEY_UP, this);
+    WidgetKeyboardEvent event(true, eKeyUp, this);
     InitKeyEvent(event, aEvent);
 
     if (aEvent->key() == Qt::Key_AltGr) {
         sAltGrModifier = false;
     }
 
-    event.keyCode = domKeyCode;
-    event.mKeyNameIndex =
-        (aEvent->text().length() && aEvent->text()[0].isPrint()) ?
-            KEY_NAME_INDEX_PrintableKey :
-            QtKeyCodeToDOMKeyNameIndex(aEvent->key());
-
     // unset the key down flag
     ClearKeyDownFlag(event.keyCode);
 
@@ -1480,7 +1264,7 @@ nsEventStatus
 nsWindow::wheelEvent(QWheelEvent* aEvent)
 {
     // check to see if we should rollup
-    WidgetWheelEvent wheelEvent(true, NS_WHEEL_WHEEL, this);
+    WidgetWheelEvent wheelEvent(true, eWheel, this);
     wheelEvent.deltaMode = nsIDOMWheelEvent::DOM_DELTA_LINE;
 
     // negative values for aEvent->delta indicate downward scrolling;
@@ -1544,21 +1328,6 @@ nsWindow::tabletEvent(QTabletEvent* aEvent)
 
 //  Helpers
 
-void
-nsWindow::InitButtonEvent(WidgetMouseEvent& aMoveEvent,
-                          QMouseEvent* aEvent,
-                          int aClickCount)
-{
-    aMoveEvent.refPoint.x = nscoord(aEvent->pos().x());
-    aMoveEvent.refPoint.y = nscoord(aEvent->pos().y());
-
-    aMoveEvent.InitBasicModifiers(aEvent->modifiers() & Qt::ControlModifier,
-                                  aEvent->modifiers() & Qt::AltModifier,
-                                  aEvent->modifiers() & Qt::ShiftModifier,
-                                  aEvent->modifiers() & Qt::MetaModifier);
-    aMoveEvent.clickCount      = aClickCount;
-}
-
 nsEventStatus
 nsWindow::DispatchEvent(WidgetGUIEvent* aEvent)
 {
@@ -1602,7 +1371,8 @@ nsWindow::DispatchDeactivateEventOnTopLevelWindow(void)
 }
 
 void
-nsWindow::DispatchResizeEvent(nsIntRect &aRect, nsEventStatus &aStatus)
+nsWindow::DispatchResizeEvent(LayoutDeviceIntRect& aRect,
+                              nsEventStatus& aStatus)
 {
     aStatus = nsEventStatus_eIgnore;
     if (mWidgetListener &&
@@ -1613,46 +1383,6 @@ nsWindow::DispatchResizeEvent(nsIntRect &aRect, nsEventStatus &aStatus)
 
 ///////////////////////////////////// OLD GECKO ECENTS need to Sort ///////////////////
 
-/* static */ bool
-isContextMenuKeyEvent(const QKeyEvent *qe)
-{
-    uint32_t kc = QtKeyCodeToDOMKeyCode(qe->key());
-    if (qe->modifiers() & (Qt::ControlModifier | Qt::AltModifier | Qt::MetaModifier)) {
-        return false;
-    }
-
-    bool isShift = qe->modifiers() & Qt::ShiftModifier;
-    return (kc == NS_VK_F10 && isShift) ||
-        (kc == NS_VK_CONTEXT_MENU && !isShift);
-}
-
-/* static */void
-InitKeyEvent(WidgetKeyboardEvent &aEvent, QKeyEvent *aQEvent)
-{
-    aEvent.InitBasicModifiers(aQEvent->modifiers() & Qt::ControlModifier,
-                              aQEvent->modifiers() & Qt::AltModifier,
-                              aQEvent->modifiers() & Qt::ShiftModifier,
-                              aQEvent->modifiers() & Qt::MetaModifier);
-    aEvent.mIsRepeat =
-        (aEvent.message == NS_KEY_DOWN || aEvent.message == NS_KEY_PRESS) &&
-        aQEvent->isAutoRepeat();
-    aEvent.time = 0;
-
-    if (sAltGrModifier) {
-        aEvent.modifiers |= (MODIFIER_CONTROL | MODIFIER_ALT);
-    }
-
-    // The transformations above and in qt for the keyval are not invertible
-    // so link to the QKeyEvent (which will vanish soon after return from the
-    // event callback) to give plugins access to hardware_keycode and state.
-    // (An XEvent would be nice but the QKeyEvent is good enough.)
-    aEvent.pluginEvent = (void *)aQEvent;
-}
-
-NS_IMPL_ISUPPORTS_INHERITED1(nsWindow, nsBaseWidget, nsISupportsWeakReference)
-
-
-
 void
 nsWindow::ClearCachedResources()
 {
@@ -1703,7 +1433,7 @@ nsWindow::PlaceBehind(nsTopLevelWidgetZPlacement  aPlacement,
 }
 
 NS_IMETHODIMP
-nsWindow::SetSizeMode(int32_t aMode)
+nsWindow::SetSizeMode(nsSizeMode aMode)
 {
     nsresult rv;
 
@@ -1771,14 +1501,14 @@ void find_first_visible_parent(QWindow* aItem, QWindow*& aVisibleItem)
 }
 
 NS_IMETHODIMP
-nsWindow::GetScreenBounds(nsIntRect &aRect)
+nsWindow::GetScreenBounds(LayoutDeviceIntRect& aRect)
 {
-    aRect = nsIntRect(nsIntPoint(0, 0), mBounds.Size());
+    aRect = LayoutDeviceIntRect(LayoutDeviceIntPoint(0, 0),
+                                LayoutDeviceIntSize::FromUnknownSize(mBounds.Size()));
     if (mIsTopLevel) {
         QPoint pos = mWidget->position();
         aRect.MoveTo(pos.x(), pos.y());
-    }
-    else {
+    } else {
         aRect.MoveTo(WidgetToScreenOffset());
     }
     LOG(("GetScreenBounds %d %d | %d %d | %d %d\n",
@@ -1788,18 +1518,6 @@ nsWindow::GetScreenBounds(nsIntRect &aRect)
     return NS_OK;
 }
 
-NS_IMETHODIMP
-nsWindow::SetForegroundColor(const nscolor &aColor)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-nsWindow::SetBackgroundColor(const nscolor &aColor)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
 NS_IMETHODIMP
 nsWindow::SetIcon(const nsAString& aIconSpec)
 {
@@ -1901,7 +1619,7 @@ nsWindow::CheckForRollup(double aMouseX, double aMouseY,
         // if we've determined that we should still rollup, do it.
         if (rollup) {
             nsIntPoint pos(aMouseX, aMouseY);
-            retVal = rollupListener->Rollup(popupsToRollup, &pos, nullptr);
+            retVal = rollupListener->Rollup(popupsToRollup, true, &pos, nullptr);
         }
     }
 
@@ -1953,7 +1671,7 @@ nsWindow::DispatchCommandEvent(nsIAtom* aCommand)
 }
 
 nsEventStatus
-nsWindow::DispatchContentCommandEvent(int32_t aMsg)
+nsWindow::DispatchContentCommandEvent(EventMessage aMsg)
 {
     WidgetContentCommandEvent event(true, aMsg, this);
 
@@ -1984,7 +1702,7 @@ GetBrandName(nsXPIDLString& brandName)
     }
 
     if (brandName.IsEmpty()) {
-        brandName.Assign(NS_LITERAL_STRING("Mozilla"));
+        brandName.AssignLiteral(MOZ_UTF16("Mozilla"));
     }
 }
 
@@ -2011,7 +1729,7 @@ nsWindow::SetWindowClass(const nsAString &xulWinType)
     }
     class_hint->res_class = ToNewCString(brandName);
     if (!class_hint->res_class) {
-        nsMemory::Free(class_hint->res_name);
+        free(class_hint->res_name);
         XFree(class_hint);
         return NS_ERROR_OUT_OF_MEMORY;
     }
@@ -2039,8 +1757,8 @@ nsWindow::SetWindowClass(const nsAString &xulWinType)
                       class_hint);
     }
 
-    nsMemory::Free(class_hint->res_class);
-    nsMemory::Free(class_hint->res_name);
+    free(class_hint->res_class);
+    free(class_hint->res_name);
     XFree(class_hint);
 #endif
 
@@ -2250,8 +1968,7 @@ void
 nsWindow::ProcessMotionEvent()
 {
     if (mMoveEvent.needDispatch) {
-        WidgetMouseEvent event(true, NS_MOUSE_MOVE, this,
-                               WidgetMouseEvent::eReal);
+        WidgetMouseEvent event(true, eMouseMove, this, WidgetMouseEvent::eReal);
 
         event.refPoint.x = nscoord(mMoveEvent.pos.x());
         event.refPoint.y = nscoord(mMoveEvent.pos.y());
